/*
 * $Id: Chart.h,v 1.4 2005/06/25 06:11:41 ninomi Exp $
 *
 *  Copyright (c) 2005, Tsujii Laboratory, The University of Tokyo.
 *  All rights reserved.
 *
 */

#ifndef __Chart_h
#define __Chart_h

#include "Edge.h"
#include "Extent.h"
#include "PreLex.h"

namespace up {

    class Chart {
    private:
        std::vector<std::vector<std::vector<Edge*> > > chart;
        std::vector<std::vector<bool> > alive;
        std::vector<std::vector<std::vector<PreLex*> > > prelex_chart;
    public:
        Chart() {}
        ~Chart() {}
        bool init(int n) {
            chart.clear();
            for(int i = 0; i < n ; i++) {
                chart.push_back(std::vector<std::vector<Edge*> >());
                for(int j = 0 ; j < n ; j++) {
                    chart[i].push_back(std::vector<Edge*>());
                }
            }
	    alive.clear();
	    for(int i = 0; i < n ; i++) {
		alive.push_back(std::vector<bool>());
		for(int j = 0 ; j < n ; j++) {
		    alive[i].push_back(true);
		}
	    }
            prelex_chart.clear();
            for(int i = 0; i < n ; i++) {
                prelex_chart.push_back(std::vector<std::vector<PreLex*> >());
                for(int j = 0 ; j < n ; j++) {
                    prelex_chart[i].push_back(std::vector<PreLex*>());
                }
            }
            return true;
        }
        void addEdge(int i, int j, Edge* e) {
            chart[i][j].push_back(e);
        }
        std::vector<Edge*>& getEdges(int i, int j) {
            return chart[i][j];
        }
        void addPreLex(int i, int j, PreLex* p) {
            prelex_chart[i][j].push_back(p);
        }
        std::vector<PreLex*>& getPreLexes(int i, int j) {
            return prelex_chart[i][j];
        }
        void clear(int n) {
            if( n >= static_cast<int>(chart.size()) ) {
                init(n+1);
            }
            for( int j = 1; j <= n ; j++ )
                for( int i = j - 1 ; i >= 0 ; i-- ) {
                    chart[i][j].clear();
                }
            for( int j = 1; j <= n ; j++ )
                for( int i = j - 1 ; i >= 0 ; i-- ) {
                    alive[i][j] = true;
                }
            for( int j = 1; j <= n ; j++ )
                for( int i = j - 1 ; i >= 0 ; i-- ) {
                    prelex_chart[i][j].clear();
                }
        }
	void makeLeftDead(int i, int j) {
	    for(int y = j - 1; y > i ; y--)
		for(int x = i - 1; x >= 0 ; x--)
		    alive[x][y] = false;
	}

	void makeRightDead(int i, int j, int n) {
	    for(int x = i + 1 ; x < j ; x++)
		for(int y = j + 1 ; y <= n ; y++)
		    alive[x][y] = false;
	}
	void makeDead(WordLattice& wordlattice, int n) {
	    WordLattice::iterator it = wordlattice.begin(), last = wordlattice.end();
	    for(; it != last ; it++) {
		ExtentBracket *exb = dynamic_cast<ExtentBracket*>(*it);
		if( exb ) {
		    makeLeftDead(exb->getLeftPosition(), exb->getRightPosition());
		    makeRightDead(exb->getLeftPosition(), exb->getRightPosition(), n);
		}
	    }
	}

        void inhibitLargeConstituents(int s, int n) {
          if(s > 0) {
            for(int i = 1; i < n - s; ++i) {
              for(int j = i + s + 1; j < n; ++j) {
                alive[i][j] = false;
              }
            }
          }
        }

        bool isAlive(int i, int j) { return alive[i][j]; }
        void setAlive(int i, int j, bool b) { alive[i][j] = b; }
    };
    
} // end  of namespace up

#endif // __Chart_h
/*
 * $Log: Chart.h,v $
 * Revision 1.4  2005/06/25 06:11:41  ninomi
 * pre-lexicon-lookup implementation
 *
 * Revision 1.3  2005/06/21 10:17:12  yusuke
 *
 * copyright 񤭤ޤ
 *
 * Revision 1.2  2005/06/16 07:25:36  yusuke
 *
 * integrate-up 硥
 * version 2.0
 *
 * Revision 1.1.2.6  2005/06/11 16:46:09  yusuke
 *
 * constituent Υη׻ѹ
 *
 * Revision 1.1.2.5  2005/06/11 15:10:50  yusuke
 *
 * constituent 礭¤Ǥ褦ˤ
 *
 * Revision 1.1.2.4  2005/06/11 13:35:52  yusuke
 *
 * 礭 constituent ʤ褦ˤȤߤ뤿ν
 *
 * Revision 1.1.2.3  2005/06/07 07:37:28  ninomi
 * Parserݥ饹ľޤ
 *
 * Revision 1.1.2.2  2005/06/05 13:14:21  ninomi
 * Ȥꤢ񴹤ޤǤФäϤ줫
 *
 * Revision 1.1.2.1  2005/06/05 10:12:18  ninomi
 * upmayz/srcβ˰ܤϤޤ
 *
 * Revision 1.5  2004/11/16 09:13:54  ninomi
 * Artistic LicenseƬˤĤޤ
 *
 * Revision 1.4  2004/06/30 07:41:48  ninomi
 * up 0.2˰ܹԤ뤿0.1θŤե
 *
 * Revision 1.3  2004/05/30 17:11:25  ninomi
 * QuickCheckǤǤEdgeInfoEdgeInfoManagerϿ(registerEdgeInfo())
 * ˺Ƥޤ
 *
 * Revision 1.2  2004/05/26 12:19:04  ninomi
 * FSPΤߤޤäƤ褷ޤ
 *
 * Revision 1.1  2004/05/20 23:24:58  ninomi
 * СǤcup, qcupưƤޤpcup, pqcupϥХȤ档
 *
 */
