/*
 * $Id: Extent.h,v 1.3 2005/06/21 10:17:12 yusuke Exp $
 *
 *  Copyright (c) 2005, Tsujii Laboratory, The University of Tokyo.
 *  All rights reserved.
 *
 */

#ifndef __Extent_h
#define __Extent_h

#include <liblilfes/structur.h>
#include "Fs.h"
#include <vector>

namespace up {

    class Extent {
    private:
        std::pair<int, int> position;
    public:
	Extent(std::pair<int, int> __position) : position(__position) {}
	virtual ~Extent() {}
        int getLeftPosition() const { return position.first; }
        int getRightPosition() const { return position.second; }
        const std::pair<int, int>& getPosition() const { return position; }
	virtual Extent* clone() = 0;
    };

    class ExtentWord : public Extent {
    private:
	Fs* word;
        ExtentWord(const ExtentWord& e)
          : Extent(e.getPosition()) {
          word = new Fs( *(e.word) );
        }
        ExtentWord& operator=(const ExtentWord& e) {
          static_cast<Extent&>(*this) = e;
          if (word) delete word;
          word = new Fs( *(e.word) );
          return *this;
        }
    public:
	ExtentWord(std::pair<int, int> __position, lilfes::FSP __word)
	    : Extent(__position) {
	    word = __word.Serialize();
	}
	~ExtentWord() { if(word) delete word; }
        Fs* getWord() { return word; }
        //void setWord(Fs* w) { word = w; }
	ExtentWord* clone() {
	    ExtentWord* ew = new ExtentWord(*this);
	    return ew;
	}
    };

    class ExtentBracket : public Extent {
    public:
	ExtentBracket(std::pair<int, int> __position)
	    : Extent(__position) {}
	~ExtentBracket() {}
	ExtentBracket* clone() { return new ExtentBracket(*this); }
    };

    class WordLattice {
    private:
	std::vector<Extent*> wordlattice;
    public:
	typedef std::vector<Extent*>::iterator iterator;
        typedef std::vector<Extent*>::const_iterator const_iterator;
	iterator begin() { return wordlattice.begin(); }
        const_iterator begin() const { return wordlattice.begin(); }
	iterator end() { return wordlattice.end(); }
        const_iterator end() const { return wordlattice.end(); }
	void push_back(Extent* e) { wordlattice.push_back(e); }
	WordLattice() {}
	WordLattice(const WordLattice& wl) {
	    WordLattice::const_iterator it = wl.begin(), last = wl.end();
	    for(; it != last; it++) {
		wordlattice.push_back((*it)->clone());
	    }
	}
        WordLattice& operator=(const WordLattice& wl) {
          std::vector<Extent*>::iterator it = wordlattice.begin(), last = wordlattice.end();
          for(; it != last; ++it) delete (*it);
          wordlattice.clear();
          std::vector<Extent*>::const_iterator wit = wl.begin(), wlast = wl.end();
          for(; wit != wlast; ++wit) {
            wordlattice.push_back((*wit)->clone());
          }
          return *this;
        }
	~WordLattice() {
	    std::vector<Extent*>::iterator it = wordlattice.begin(), last = wordlattice.end();
	    for(; it != last ; it++)
		delete (*it);
	}
    };
    
} // end of namespace up
#endif // __Extent_h

/*
 * $Log: Extent.h,v $
 * Revision 1.3  2005/06/21 10:17:12  yusuke
 *
 * copyright 񤭤ޤ
 *
 * Revision 1.2  2005/06/16 07:25:36  yusuke
 *
 * integrate-up 硥
 * version 2.0
 *
 * Revision 1.1.2.1  2005/06/05 10:12:18  ninomi
 * upmayz/srcβ˰ܤϤޤ
 *
 * Revision 1.9  2004/11/16 09:13:54  ninomi
 * Artistic LicenseƬˤĤޤ
 *
 * Revision 1.8  2004/07/13 14:40:30  yusuke
 *
 * ExtentWord::clone ꡼Ƥ
 *
 * Revision 1.7  2004/07/09 02:11:53  yusuke
 *
 * Parser::getWordLattice 
 * iterative parsing ǡbeam ѤƤʤȤ
 * parseLexiconLookup ʤ褦ˤ
 *
 * Revision 1.6  2004/06/30 07:41:48  ninomi
 * up 0.2˰ܹԤ뤿0.1θŤե
 *
 * Revision 1.5  2004/05/16 09:06:04  ninomi
 * 󥯤˼ԤƤΤޤ(GrammarIFinit()貾۴
 * ˤʤäƤʤä)
 *
 * Revision 1.4  2004/05/14 12:41:30  ninomi
 * ѥ뤬Ȥ褦ˤʤޤ
 *
 * Revision 1.3  2004/05/14 09:55:56  ninomi
 * ȤꤢncupɬפʤΤҤȤȤ·ޤ
 *
 * Revision 1.2  2004/05/14 05:01:30  ninomi
 * GrammarIFNaiveȤ̾ΤGrammarIFBasicȤ̾Τˤޤ
 *
 * Revision 1.1  2004/05/14 04:51:34  ninomi
 * LatticeNodeExtentȤ̾Τѹޤ
 *
 * Revision 1.2  2004/05/14 04:33:29  ninomi
 * Ǥ
 *
 * Revision 1.1  2004/05/13 07:07:23  ninomi
 * ʥեɲǤ
 *
 */

