/*
 * $Id: Fom.cc,v 1.3 2005/06/21 10:17:12 yusuke Exp $
 *
 *  Copyright (c) 2005, Tsujii Laboratory, The University of Tokyo.
 *  All rights reserved.
 *
 */

#include <iostream>
#include "Fom.h"
#include "liblilfes/utility.h"

namespace up {

    using namespace lilfes;

    void FomManager::clear() {
	edge_fom.clear();
	link_fom.clear();
    }

    void FomManager::setEdgeFom(eserial esn, double fom) {
	if(esn >= edge_fom.size())
	    edge_fom.resize(esn+1);
	edge_fom[esn] = fom;
    }
    double FomManager::getEdgeFom(eserial esn) {
	if(esn >= edge_fom.size())
	    return -999999;
	return edge_fom[esn];
    }
    void FomManager::setLinkFom(lserial isn, double fom) {
	if(isn >= link_fom.size())
	    link_fom.resize(isn+1);
	link_fom[isn] = fom;
    }
    double FomManager::getLinkFom(lserial isn) {
	if(isn >= link_fom.size())
	    return 999;
	return link_fom[isn];
    }

    bool FomManager::init(machine* m) {
	mach = m;
        
	lilfes::module *up_fom;
        
	// set modules
	if(! (up_fom = lilfes::module::SearchModule("mayz:parser") )) {
          RUNERR( "module not found: mayz:parser" );
          return false;
	}
        
	// set procedures
	nf_fom_root = 2;
	nf_fom_binary = 6;
	nf_fom_unary = 5;
	nf_fom_terminal = 4;
	nf_fom_lexical_entry = 3;

	fom_root = lilfes::prepare_proc(up_fom, "fom_root", nf_fom_root);
	fom_binary = lilfes::prepare_proc(up_fom, "fom_binary", nf_fom_binary);
	fom_unary = lilfes::prepare_proc(up_fom, "fom_unary", nf_fom_unary);
	fom_terminal = lilfes::prepare_proc(up_fom, "fom_terminal", nf_fom_terminal);
	fom_lexical_entry = lilfes::prepare_proc(up_fom, "fom_lexical_entry", nf_fom_lexical_entry);
  
	if(! fom_root) {
          RUNERR( "predicate `fom_root/" << nf_fom_root << "' is not defined" );
          return false;
	}
	if(! fom_binary) {
          RUNERR( "predicate `fom_binary/" << nf_fom_binary << "' is not defined" );
          return false;
	}
	if(! fom_unary) {
          RUNERR( "predicate `fom_unary/" << nf_fom_unary << "' is not defined" );
          return false;
	}
	if(! fom_terminal) {
          RUNERR( "predicate `fom_terminal/" << nf_fom_terminal << "' is not defined" );
          return false;
	}
	if(! fom_lexical_entry) {
          RUNERR( "predicate `fom_lexical_entry/" << nf_fom_lexical_entry << "' is not defined" );
          return false;
	}
        
        is_open = true;
        
	return true;
    }
    bool FomManager::term() {
        if(is_open) {
            is_open = false;
        }
        return true;
    }

    bool FomManager::fomRoot(FSP sign, FSP fom) {
        P2("FomManager::fomRoot");
        procthunk *pt = fom_root->GetThunk();
        if(! pt) return false;
        mach->SetRP(nf_fom_root);
        mach->WriteLocal(0, PTR2c(sign.GetAddress()));
        mach->WriteLocal(1, PTR2c(fom.GetAddress()));
        pt->exec(*mach);
        if( mach->GetIP() == NULL ) return false;
	return mach->Execute(mach->GetIP());
    }
    bool FomManager::fomBinary(FSP name, FSP left, FSP right, FSP mother, FSP sign_plus, FSP fom) {
        P2("FomManager::fomBinary");
        procthunk *pt = fom_binary->GetThunk();
        if(! pt) return false;
        mach->SetRP(nf_fom_binary);
        mach->WriteLocal(0, PTR2c(name.GetAddress()));
        mach->WriteLocal(1, PTR2c(left.GetAddress()));
        mach->WriteLocal(2, PTR2c(right.GetAddress()));
        mach->WriteLocal(3, PTR2c(mother.GetAddress()));
        mach->WriteLocal(4, PTR2c(sign_plus.GetAddress()));
        mach->WriteLocal(5, PTR2c(fom.GetAddress()));
        pt->exec(*mach);
        if( mach->GetIP() == NULL ) return false;
	return mach->Execute(mach->GetIP());
    }
    bool FomManager::fomUnary(FSP name, FSP dtr, FSP mother, FSP sign_plus, FSP fom) {
        P2("FomManager::fomUnary");
        procthunk *pt = fom_unary->GetThunk();
        if(! pt) return false;
        mach->SetRP(nf_fom_unary);
        mach->WriteLocal(0, PTR2c(name.GetAddress()));
        mach->WriteLocal(1, PTR2c(dtr.GetAddress()));
        mach->WriteLocal(2, PTR2c(mother.GetAddress()));
        mach->WriteLocal(3, PTR2c(sign_plus.GetAddress()));
        mach->WriteLocal(4, PTR2c(fom.GetAddress()));
        pt->exec(*mach);
        if( mach->GetIP() == NULL ) return false;
	return mach->Execute(mach->GetIP());
    }
    bool FomManager::fomTerminal(FSP lex_name, FSP sign, FSP sign_plus, FSP fom) {
        P2("FomManager::fomTerminal");
        procthunk *pt = fom_terminal->GetThunk();
        if(! pt) return false;
        mach->SetRP(nf_fom_terminal);
        mach->WriteLocal(0, PTR2c(lex_name.GetAddress()));
        mach->WriteLocal(1, PTR2c(sign.GetAddress()));
        mach->WriteLocal(2, PTR2c(sign_plus.GetAddress()));
        mach->WriteLocal(3, PTR2c(fom.GetAddress()));
        pt->exec(*mach);
        if( mach->GetIP() == NULL ) return false;
	return mach->Execute(mach->GetIP());
    }
    bool FomManager::fomLexicalEntry(FSP word, FSP lex_name, FSP fom) {
        P2("FomManager::fomLexicalEntry");
        procthunk *pt = fom_lexical_entry->GetThunk();
        if(! pt) return false;
        mach->SetRP(nf_fom_lexical_entry);
        mach->WriteLocal(0, PTR2c(word.GetAddress()));
        mach->WriteLocal(1, PTR2c(lex_name.GetAddress()));
        mach->WriteLocal(2, PTR2c(fom.GetAddress()));
        pt->exec(*mach);
        if( mach->GetIP() == NULL ) return false;
	return mach->Execute(mach->GetIP());
    }
}

/*
 * $Log: Fom.cc,v $
 * Revision 1.3  2005/06/21 10:17:12  yusuke
 *
 * copyright 񤭤ޤ
 *
 * Revision 1.2  2005/06/16 07:25:36  yusuke
 *
 * integrate-up 硥
 * version 2.0
 *
 * Revision 1.1.2.7  2005/06/11 04:05:02  yusuke
 *
 * cout -> *lilfes::output_stream
 * cerr -> *lilfes::error_stream
 *
 * Revision 1.1.2.6  2005/06/09 00:16:10  yusuke
 *
 * extract_???_event  fom_??? ˤ碌ƥ󥿥եѹ
 *
 * Revision 1.1.2.5  2005/06/08 05:59:42  ninomi
 * P2ץեĤʤޤ
 *
 * Revision 1.1.2.4  2005/06/06 16:35:20  yusuke
 *
 * enju ư褦ˤʤޤ
 *
 * Revision 1.1.2.3  2005/06/06 08:53:41  ninomi
 * IPTrailStackطľޤlilfespredicate callʤޤ
 * quick checkidҤˤΤnameǤҤ褦ˤޤ
 *
 * Revision 1.1.2.2  2005/06/05 14:10:48  ninomi
 * ѥ뤬Ȥ褦ˤʤޤ
 *
 * Revision 1.1.2.1  2005/06/05 10:12:18  ninomi
 * upmayz/srcβ˰ܤϤޤ
 *
 * Revision 1.9  2005/02/26 12:18:16  yusuke
 *
 * edge_info γΨȡfom_equivalent_reduction 裳¸ơ
 * edge_info_fom 褦ˤ
 *
 * Revision 1.8  2004/11/16 09:14:07  ninomi
 * Artistic LicenseƬˤĤޤ
 *
 * Revision 1.7  2004/09/13 15:04:25  yusuke
 *
 * fom_init ƤʤȤ segmentation fault ˤʤä
 *
 * Revision 1.6  2004/07/06 13:05:57  ninomi
 * Parser::term()ɲäޤlilfesterminator::PerformAll()¹
 * ɬ¹ԤʤȤʤ
 *
 * Revision 1.5  2004/07/02 06:40:41  ninomi
 * inittermޤΥХȤޤ
 *
 * Revision 1.4  2004/07/01 13:05:40  ninomi
 * FomManagerGrammaropenʾ֤ΤȤΤdestructorlilfes???term
 * ¹Ԥ褦ˤޤ
 *
 * Revision 1.3  2004/07/01 09:55:45  ninomi
 * FomManager::~FomManager()Grammar::~Grammar()lilfesҸfom_termgrammar_term򤽤줾ƤӽФ褦ˤޤ
 *
 * Revision 1.2  2004/07/01 09:24:09  ninomi
 * FomManager::init()Grammar::init()fom_initgrammar_init
 * lilfesҸƽФ褦ˤʤޤ
 *
 * Revision 1.1  2004/06/30 07:50:29  ninomi
 * up 0.2release versionǤ
 *
 */
