/*
 * $Id: Fom.h,v 1.3 2005/06/21 10:17:12 yusuke Exp $
 *
 *  Copyright (c) 2005, Tsujii Laboratory, The University of Tokyo.
 *  All rights reserved.
 *
 */

#ifndef __Fom_h
#define __Fom_h

#include <liblilfes/machine.h>
#include <liblilfes/proc.h>
#include <liblilfes/structur.h>
#include "Edge.h"

namespace up {

    class FomManager {
    private:
        bool is_open;
        
        lilfes::machine *mach;
        
        lilfes::procedure *fom_root;
        lilfes::procedure *fom_binary;
        lilfes::procedure *fom_unary;
        lilfes::procedure *fom_terminal;
        lilfes::procedure *fom_lexical_entry;
  
        int nf_fom_root;
        int nf_fom_binary;
        int nf_fom_unary;
        int nf_fom_terminal;
        int nf_fom_lexical_entry;
  
        std::vector<double> edge_fom;
	std::vector<double> link_fom;
//         std::vector< std::vector< lilfes::cell > > link_reduction;

    public:
        FomManager() : is_open(false) {}
        ~FomManager() {
            term();
        }

        bool init(lilfes::machine *m);
        bool term();
	void clear();
	
	void setEdgeFom(eserial esn, double fom);
	double getEdgeFom(eserial esn);
	void setLinkFom(lserial isn, double fom);
	double getLinkFom(lserial isn);

//         void setLinkReduction(lserial isn, lilfes::FSP);
//         const std::vector< lilfes::cell >& getLinkReduction(lserial isn) const;

	bool fomRoot(lilfes::FSP, lilfes::FSP);
        bool fomBinary(lilfes::FSP, lilfes::FSP, lilfes::FSP, lilfes::FSP, lilfes::FSP, lilfes::FSP);
        bool fomUnary(lilfes::FSP, lilfes::FSP, lilfes::FSP, lilfes::FSP, lilfes::FSP);
	bool fomTerminal(lilfes::FSP, lilfes::FSP, lilfes::FSP, lilfes::FSP);
	bool fomLexicalEntry(lilfes::FSP, lilfes::FSP, lilfes::FSP);

     };
    
} // end of namespace up

#endif // Fom_h

/*
 * $Log: Fom.h,v $
 * Revision 1.3  2005/06/21 10:17:12  yusuke
 *
 * copyright 񤭤ޤ
 *
 * Revision 1.2  2005/06/16 07:25:36  yusuke
 *
 * integrate-up 硥
 * version 2.0
 *
 * Revision 1.1.2.5  2005/06/09 00:16:10  yusuke
 *
 * extract_???_event  fom_??? ˤ碌ƥ󥿥եѹ
 *
 * Revision 1.1.2.4  2005/06/06 16:35:20  yusuke
 *
 * enju ư褦ˤʤޤ
 *
 * Revision 1.1.2.3  2005/06/06 08:53:41  ninomi
 * IPTrailStackطľޤlilfespredicate callʤޤ
 * quick checkidҤˤΤnameǤҤ褦ˤޤ
 *
 * Revision 1.1.2.2  2005/06/05 14:10:48  ninomi
 * ѥ뤬Ȥ褦ˤʤޤ
 *
 * Revision 1.1.2.1  2005/06/05 10:12:18  ninomi
 * upmayz/srcβ˰ܤϤޤ
 *
 * Revision 1.7  2005/02/26 12:18:12  yusuke
 *
 * edge_info γΨȡfom_equivalent_reduction 裳¸ơ
 * edge_info_fom 褦ˤ
 *
 * Revision 1.6  2004/11/16 09:13:54  ninomi
 * Artistic LicenseƬˤĤޤ
 *
 * Revision 1.5  2004/07/02 06:40:35  ninomi
 * inittermޤΥХȤޤ
 *
 * Revision 1.4  2004/07/01 13:05:35  ninomi
 * FomManagerGrammaropenʾ֤ΤȤΤdestructorlilfes???term
 * ¹Ԥ褦ˤޤ
 *
 * Revision 1.3  2004/07/01 09:55:33  ninomi
 * FomManager::~FomManager()Grammar::~Grammar()lilfesҸfom_termgrammar_term򤽤줾ƤӽФ褦ˤޤ
 *
 * Revision 1.2  2004/07/01 09:23:49  ninomi
 * FomManager::init()Grammar::init()fom_initgrammar_init
 * lilfesҸƽФ褦ˤʤޤ
 *
 * Revision 1.1  2004/06/30 07:50:09  ninomi
 * up 0.2release versionǤ
 *
 */
