/**********************************************************************
 *
 *  Copyright (c) 2005, Tsujii Laboratory, The University of Tokyo.
 *  All rights reserved.
 *
 *  @file ForestMaker.h
 *  @version Time-stamp: <2005-06-21 19:06:44 yusuke>
 *  Implementation of forestmaker
 *
 **********************************************************************/

#ifndef MAYZ_FOREST_MAKER_H

#define MAYZ_FOREST_MAKER_H

#include "config_mayz.h"
#include <liblilfes/lilfes.h>
#include "ParserCky.h"
#include "FMGrammar.h"
#include "Derivation.h"
#include "ParseTree.h"

namespace mayz {

  class ForestMakerException : public std::exception {
  private:
    std::string message;

  public:
    ForestMakerException( const char* m ) : message( m ) {
    }
    ~ForestMakerException() throw () {}
    const char* what() const throw () { return message.c_str(); }
  };

  class ForestMaker : public up::ParserCky {
  private:
    std::string model_name;
    std::vector< bool > visit_mark;
    std::ostream* output_stream;
    std::ostream* reference_stream;
    double reference_probability;

    lilfes::procedure* convert_derivation;

    lilfes::procedure* extract_terminal_event;
    lilfes::procedure* extract_unary_event;
    lilfes::procedure* extract_binary_event;
    lilfes::procedure* extract_root_event;

    lilfes::procedure* extract_terminal_event_feature_value;
    lilfes::procedure* extract_unary_event_feature_value;
    lilfes::procedure* extract_binary_event_feature_value;
    lilfes::procedure* extract_root_event_feature_value;

    lilfes::procedure* reference_prob_terminal;
    lilfes::procedure* reference_prob_unary;
    lilfes::procedure* reference_prob_binary;
    lilfes::procedure* reference_prob_root;

  public:
    virtual bool outputTerminalEvents( lilfes::FSP lexname, lilfes::FSP sign, lilfes::FSP sign_plus, std::ostream& output_file );
    virtual bool outputUnaryEvents( lilfes::FSP schema, lilfes::FSP dtr, lilfes::FSP mother, lilfes::FSP sign_plus, std::ostream& output_file );
    virtual bool outputBinaryEvents( lilfes::FSP schema, lilfes::FSP ldtr, lilfes::FSP rdtr, lilfes::FSP mother, lilfes::FSP sign_plus, std::ostream& output_file );
    virtual bool outputRootEvents( lilfes::FSP sign, std::ostream& output_file );

    virtual bool terminalReference( lilfes::FSP lexname, lilfes::FSP sign, lilfes::FSP sign_plus, double& ref );
    virtual bool unaryReference( lilfes::FSP schema_name, lilfes::FSP dtr, lilfes::FSP mother, lilfes::FSP sign_plus, double& ref );
    virtual bool binaryReference( lilfes::FSP schema_name, lilfes::FSP left_dtr, lilfes::FSP right_dtr, lilfes::FSP mother, lilfes::FSP sign_plus, double& ref );
    virtual bool rootReference( lilfes::FSP sign, double& ref );

  protected:
    virtual bool outputParseTreeDtrs( ParseTree parse_tree );
    virtual bool outputParseTreeRoot( ParseTree parse_tree );

    virtual bool outputForestConjTerm( up::Edge* edge, up::LinkTerminal* edge_link, int id );
    virtual bool outputForestConjUnary( up::Edge* edge, up::LinkUnary* edge_link, int id );
    virtual bool outputForestConjBinary( up::Edge* edge, up::LinkBinary* edge_link, int id );
    virtual bool outputForestDisj( up::Edge* edge );
    virtual bool outputForestRoot();

  public:
    ForestMaker( const std::string& n )
      : up::ParserCky(),
        model_name( n ),
        output_stream( NULL ),
        reference_stream( NULL ),
        reference_probability( 0.0 ) {
    }
    virtual ~ForestMaker() {
    }

    bool init( lilfes::machine* m, up::Grammar* g );

    virtual bool makeWordLattice( Derivation derivation, up::WordLattice& wordlattice );
    virtual bool derivationToParseTree( Derivation derivation, ParseTree parse_tree );
    virtual bool outputParseTree( ParseTree parse_tree, std::ostream& output_file, std::ostream& reference_file );
    virtual bool outputParseTree( ParseTree parse_tree, std::ostream& output_file );
    virtual bool outputForest( std::ostream& output_file, std::ostream& reference_file );
    virtual bool outputForest( std::ostream& output_file );

  };

}

#endif // MAYZ_FOREST_MAKER_H
