/*
 * $Id: Fsp.cc,v 1.3 2005/06/21 10:17:12 yusuke Exp $
 *
 *  Copyright (c) 2005, Tsujii Laboratory, The University of Tokyo.
 *  All rights reserved.
 *
 */

#include <liblilfes/builtin.h>
#include "Fsp.h"

namespace up {

    using namespace lilfes;

    bool FspMaker::init(lilfes::machine *m) {
	mach = m;
        module *up_interface;
	// set modules
        if(! (up_interface = module::SearchModule("mayz:parser") )) {
          RUNERR( "module not found: mayz:parser" );
          return false;
	}
	// set type
        if(! (terminal = up_interface->Search("terminal")) ) {
          RUNERR( "type not found: `terminal' (maybe inconsistency of parser version)" );
          return false;
	}
        if(! (nonterminal_unary = up_interface->Search("nonterminal_unary")) ) {
          RUNERR( "type not found: `nonterminal_unary' (maybe inconsistency of parser version)" );
          return false;
	}
	if(! (nonterminal_binary = up_interface->Search("nonterminal_binary")) ) {
          RUNERR( "type not found: 'nonterminal_binary' (maybe inconsistency of parser version)" );
          return false;
        }
        if(! (extent = up_interface->Search("extent")) ) {
          RUNERR( "type not found: 'extent' (maybe inconsistency of parser version)" );
          return false;
        }
        if(! (extent_bracket = up_interface->Search("extent_bracket")) ) {
          RUNERR( "type not found:  'extent_bracket' (maybe inconsistency of parser version)" );
          return false;
        }
        if(! (extent_word = up_interface->Search("extent_word")) ) {
          RUNERR( "type not found: 'extent_word' (maybe inconsistency of parser version)" );
          return false;
        }

            // set feature
        if(! (f_sign_plus = feature::Search(up_interface->Search("SIGN_PLUS\\"))) ) {
          RUNERR( "feature not found: `SIGN_PLUS\\' (maybe inconsistency of parser version)" );
          return false;
        }
        if(! (f_lex_name = feature::Search(up_interface->Search("LEX_NAME\\"))) ) {
          RUNERR( "feature not found: `LEX_NAME\\' (maybe inconsistency of parser version)" );
          return false;
        }
        if(! (f_applied_schema = feature::Search(up_interface->Search("APPLIED_SCHEMA\\"))) ) {
          RUNERR( "feature not found: `APPLIED_SCHEMA\\' (maybe inconsistency of parser version)" );
          return false;
        }
        if(! (f_l_dtr = feature::Search(up_interface->Search("L_DTR\\"))) ) {
          RUNERR( "feature not found: `L_DTR\\' (maybe inconsistency of parser version)" );
          return false;
        }
        if(! (f_r_dtr = feature::Search(up_interface->Search("R_DTR\\"))) ) {
          RUNERR( "feature not found: `R_DTR\\' (maybe inconsistency of parser version)" );
          return false;
        }
        if(! (f_u_dtr = feature::Search(up_interface->Search("U_DTR\\"))) ) {
          RUNERR( "feature not found: `U_DTR\\' (maybe inconsistency of parser version)" );
          return false;
        }
        if(! (f_left_position = feature::Search(up_interface->Search("left_position\\"))) ) {
          RUNERR( "feature not found: `left_position\\' (maybe inconsistency of parser version)" );
          return false;
        }
        if(! (f_right_position = feature::Search(up_interface->Search("right_position\\"))) ) {
          RUNERR( "feature not found: `right_position\\' (maybe inconsistency of parser version" );
          return false;
        }
        if(! (f_word = feature::Search(up_interface->Search("word\\"))) ) {
          RUNERR( "feature not found: `word\\' (maybe inconsistency of parser version" );
          return false;
        }

        return true;
    }

    lilfes::FSP FspMaker::makeFSP(Edge* e) {
	return FSP(mach, e->getSign());
    }
    lilfes::FSP FspMaker::makeFSP(Link* link) {
	LinkBinary* b = dynamic_cast<LinkBinary*>(link);
	if(b) return makeFSP(b);
	LinkUnary* u =  dynamic_cast<LinkUnary*>(link);
	if(u) return makeFSP(u);
	LinkTerminal* t = dynamic_cast<LinkTerminal*>(link);
	if(t) return makeFSP(t);
	return FSP();
    }
    lilfes::FSP FspMaker::makeFSP(LinkBinary* link) {

        Fs* sign_plus = link->getSignPlus();
        Fs* applied_schema = link->getAppliedSchema();
// 	int l = link->getLeftPosition();
// 	int r = link->getRightPosition();
        int lsn = link->getLDtr()->getSerialNo();
        int rsn = link->getRDtr()->getSerialNo();
//         int lcl = link->getLDtr()->getLeftPosition();
//         int lcr = link->getLDtr()->getRightPosition();
//         int rcl = link->getRDtr()->getLeftPosition();
//         int rcr = link->getRDtr()->getRightPosition();
        
        lilfes::FSP y(mach);
        y.Coerce(nonterminal_binary);
        y.Follow(f_sign_plus).Unify(lilfes::FSP(mach, sign_plus));
//         y.Follow(f_cell).Coerce(t_comma);
//         y.Follow(f_cell).Follow(f_arg[1]).Unify(lilfes::FSP(mach, (mint) l));
//         y.Follow(f_cell).Follow(f_arg[2]).Unify(lilfes::FSP(mach, (mint) r));
        y.Follow(f_applied_schema).Unify(lilfes::FSP(mach, applied_schema));
        y.Follow(f_l_dtr).Unify(lilfes::FSP(mach, (mint) lsn));
        y.Follow(f_r_dtr).Unify(lilfes::FSP(mach, (mint) rsn));
//         y.Follow(f_l_cell).Coerce(t_comma);
//         y.Follow(f_l_cell).Follow(f_arg[1]).Unify(lilfes::FSP(mach, (mint) lcl));
//         y.Follow(f_l_cell).Follow(f_arg[2]).Unify(lilfes::FSP(mach, (mint) lcr));
//         y.Follow(f_r_cell).Coerce(t_comma);
//         y.Follow(f_r_cell).Follow(f_arg[1]).Unify(lilfes::FSP(mach, (mint) rcl));
//         y.Follow(f_r_cell).Follow(f_arg[2]).Unify(lilfes::FSP(mach, (mint) rcr));
        return y;
    }
    lilfes::FSP FspMaker::makeFSP(LinkUnary* link) {

        Fs* sign_plus = link->getSignPlus();
        Fs* applied_schema = link->getAppliedSchema();
// 	int l = link->getLeftPosition();
// 	int r = link->getRightPosition();
        int usn = link->getUDtr()->getSerialNo();
//         int ul = link->getUCell().first;
//         int ur = link->getUCell().second;
        
        lilfes::FSP y(mach);
        y.Coerce(nonterminal_unary);
        y.Follow(f_sign_plus).Unify(lilfes::FSP(mach, sign_plus));
//         y.Follow(f_cell).Coerce(t_comma);
//         y.Follow(f_cell).Follow(f_arg[1]).Unify(lilfes::FSP(mach, (mint) l));
//         y.Follow(f_cell).Follow(f_arg[2]).Unify(lilfes::FSP(mach, (mint) r));
        y.Follow(f_applied_schema).Unify(lilfes::FSP(mach, applied_schema));
        y.Follow(f_u_dtr).Unify(lilfes::FSP(mach, (mint) usn));
//         y.Follow(f_u_cell).Coerce(t_comma);
//         y.Follow(f_u_cell).Follow(f_arg[1]).Unify(lilfes::FSP(mach, (mint) ul));
//         y.Follow(f_u_cell).Follow(f_arg[2]).Unify(lilfes::FSP(mach, (mint) ur));
        return y;
    }
    lilfes::FSP FspMaker::makeFSP(LinkTerminal* link) {

        Fs* sign_plus = link->getSignPlus();
        Fs* lex_name = link->getLexName();
// 	int l = link->getLeftPosition();
// 	int r = link->getRightPosition();
//         int edge_id = link->getMyEdgeSerialNo();
        
        lilfes::FSP y(mach);
        y.Coerce(terminal);
        y.Follow(f_sign_plus).Unify(lilfes::FSP(mach, sign_plus));
//         y.Follow(f_cell).Coerce(t_comma);
//         y.Follow(f_cell).Follow(f_arg[1]).Unify(lilfes::FSP(mach, (mint) l));
//         y.Follow(f_cell).Follow(f_arg[2]).Unify(lilfes::FSP(mach, (mint) r));
//         y.Follow(f_leaf_cell).Coerce(t_comma);
//         y.Follow(f_leaf_cell).Follow(f_arg[1]).Unify(lilfes::FSP(mach, (mint) l));
//         y.Follow(f_leaf_cell).Follow(f_arg[2]).Unify(lilfes::FSP(mach, (mint) r));
        y.Follow(f_lex_name).Unify(lilfes::FSP(mach, lex_name));
//         y.Follow(f_leaf_id).Unify(lilfes::FSP(mach, (mint) edge_id));
        return y;
    }
    lilfes::FSP FspMaker::makeFSP(Extent* extent) {
        ExtentWord *exw = dynamic_cast<ExtentWord*>(extent);
	if(exw) {
            FSP r(mach); r.Coerce(extent_word);
            r.Follow(f_left_position).Unify(FSP(mach, (mint) exw->getLeftPosition()));
            r.Follow(f_right_position).Unify(FSP(mach, (mint) exw->getRightPosition()));
            r.Follow(f_word).Unify(FSP(mach, exw->getWord()));
            return r;
            
        }
	ExtentBracket *exb = dynamic_cast<ExtentBracket*>(extent);
	if(exb) {
            FSP r(mach); r.Coerce(extent_bracket);
            r.Follow(f_left_position).Unify(FSP(mach, (mint) exb->getLeftPosition()));
            r.Follow(f_right_position).Unify(FSP(mach, (mint) exb->getRightPosition()));
            return r;
        }
        return lilfes::FSP();
    }
//     lilfes::FSP FspMaker::makeFSP(Link* einfo, FomManager* fm) {
//       if ( link_fom == NULL ) return FSP();
//       FSP y(mach);
//       float fom = fm->getLinkFom( einfo->getSerialNo() );
//       const std::vector< cell >& reduction = fm->getLinkReduction( einfo->getSerialNo() );
//       FSP reduction2( mach, reduction );
//       y.Coerce( link_fom );
//       y.Follow( f_link_fom ).Unify( fom );
//       y.Follow( f_sign_reduction ).Unify( reduction2 );
//       return y;
//     }
    Extent* FspMaker::newExtent(lilfes::FSP fs) {

        if( fs.GetType()->IsSubType(extent_word) ) {
            FSP lp = fs.Follow(f_left_position);
            FSP rp = fs.Follow(f_right_position);
            FSP wd = fs.Follow(f_word);
                
            if(! lp.IsInteger() ) { return 0; }// Error
            if(! rp.IsInteger() ) { return 0; }// Error

            return new ExtentWord(std::pair<int, int>(lp.ReadInteger(), rp.ReadInteger()), wd);
            
        } else if( fs.GetType()->IsSubType(extent_bracket) ) {
            FSP lp = fs.Follow(f_left_position);
            FSP rp = fs.Follow(f_right_position);
                
            if(! lp.IsInteger() ) { return 0; }// Error
            if(! rp.IsInteger() ) { return 0; }// Error

            return new ExtentBracket(std::pair<int, int>(lp.ReadInteger(), rp.ReadInteger()));
        }
        return 0;
    }
} //end of namespace up

/*
 * $Log: Fsp.cc,v $
 * Revision 1.3  2005/06/21 10:17:12  yusuke
 *
 * copyright 񤭤ޤ
 *
 * Revision 1.2  2005/06/16 07:25:36  yusuke
 *
 * integrate-up 硥
 * version 2.0
 *
 * Revision 1.1.2.7  2005/06/11 04:05:02  yusuke
 *
 * cout -> *lilfes::output_stream
 * cerr -> *lilfes::error_stream
 *
 * Revision 1.1.2.6  2005/06/06 14:56:05  yusuke
 *
 * get_parser_mode, get_parse_status ʳϼ
 *
 * Revision 1.1.2.5  2005/06/06 14:11:23  yusuke
 *
 * ParserCky Ϥۤư褦ˤʤޤ
 *
 * Revision 1.1.2.4  2005/06/06 12:01:02  yusuke
 *
 * Ȥꤢ sample_hpsg  warning Фʤ褦ˤޤ
 *
 * Revision 1.1.2.3  2005/06/06 09:00:18  ninomi
 * Fsp.ccextentleft_pos, right_posleft_position, right_positionѹ
 * ޤ
 *
 * Revision 1.1.2.2  2005/06/05 14:10:48  ninomi
 * ѥ뤬Ȥ褦ˤʤޤ
 *
 * Revision 1.1.2.1  2005/06/05 10:12:18  ninomi
 * upmayz/srcβ˰ܤϤޤ
 *
 * Revision 1.3  2005/02/26 12:18:16  yusuke
 *
 * edge_info γΨȡfom_equivalent_reduction 裳¸ơ
 * edge_info_fom 褦ˤ
 *
 * Revision 1.2  2004/11/16 09:14:07  ninomi
 * Artistic LicenseƬˤĤޤ
 *
 * Revision 1.1  2004/06/30 07:50:29  ninomi
 * up 0.2release versionǤ
 *
 */
