/*
 * $Id: Grammar.cc,v 1.3 2005/06/21 10:17:12 yusuke Exp $
 *
 *  Copyright (c) 2005, Tsujii Laboratory, The University of Tokyo.
 *  All rights reserved.
 *
 */

#include <liblilfes/utility.h>
#include "Grammar.h"
#include <iostream>

namespace up {

    using namespace lilfes;
    
    
    bool Grammar::init(lilfes::machine* m) {
	if(is_open) return true;

	mach = m;
        module *up_interface;
	// set modules
        if(! (up_interface = module::SearchModule("mayz:parser") )) {
          RUNERR( "module not found: mayz:parser" );
          return false;
	}
	// set procedure
        sentence_to_word_lattice = lilfes::prepare_proc(up_interface, "sentence_to_word_lattice", 2);
        lexical_entry = lilfes::prepare_proc(up_interface, "lexical_entry", 2);
        lexical_entry_sign = lilfes::prepare_proc(up_interface, "lexical_entry_sign", 2);
        id_schema_unary = lilfes::prepare_proc(up_interface, "id_schema_unary", 4);
        id_schema_binary = lilfes::prepare_proc(up_interface, "id_schema_binary", 5);
        reduce_sign = lilfes::prepare_proc(up_interface, "reduce_sign", 3);
        root_sign = lilfes::prepare_proc(up_interface, "root_sign", 1);
        
        nf_sentence_to_word_lattice = 2;
        nf_lexical_entry = 2;
        nf_lexical_entry_sign = 2;
        nf_id_schema_unary = 4;
        nf_id_schema_binary = 5;
        nf_reduce_sign = 3;
        nf_root_sign = 1;

	if(! sentence_to_word_lattice) {
          RUNERR( "predicate not defined: `sentence_to_word_lattice/" << nf_sentence_to_word_lattice << "'" );
          return false;
	}
	if(! lexical_entry) {
          RUNERR( "predicate not defined: `lexical_entry/" << nf_lexical_entry << "'" );
          return false;
	}
	if(! lexical_entry_sign) {
          RUNERR( "predicate not defined: `lexical_entry_sign/" << nf_lexical_entry_sign << "'" );
          return false;
	}
        // unary schema may not be defined
// 	if(! id_schema_unary) {
// 	    std::cout << "predicate `id_schema_unary' is not defined" << std::endl;
// 	    return false;
// 	}
	if(! id_schema_binary) {
          RUNERR( "predicate not defined: `id_schema_binary/" << nf_id_schema_binary << "'" );
          return false;
	}
	if(! reduce_sign) {
          RUNERR( "predicate not defined: `reduce_sign/" << nf_reduce_sign << "'" );
          return false;
	}
        if(! root_sign) {
          RUNERR( "predicate not defined: `root_sign/" << nf_root_sign << "'" );
          return false;
        }

        // unary grammar check
        code *oldIP = mach->GetIP();
        core_p TrailP = mach->SetTrailPoint(NULL);
	is_unary = idSchemaUnary(FSP(mach), FSP(mach), FSP(mach), FSP(mach));
	mach->TrailBack(TrailP);
	mach->SetIP(oldIP);

	is_open = true;
        return true;
    }
    bool Grammar::term() {
        if(is_open) {
            is_open = false;
        }
        return true;
    }
    
    bool Grammar::sentence2WordLattice(FSP sentence, FSP wordlattice) {
        P2("Grammar::sentence2WordLattice");
        
        procthunk *pt = sentence_to_word_lattice->GetThunk();
        if(! pt) return false;

        mach->SetRP(nf_sentence_to_word_lattice);

        mach->WriteLocal(0, PTR2c(sentence.GetAddress()));
        mach->WriteLocal(1, PTR2c(wordlattice.GetAddress()));

        pt->exec(*mach);

        if( mach->GetIP() == NULL ) return false;
	return mach->Execute(mach->GetIP());
    }
    
    bool Grammar::lexicalEntry(FSP word, FSP lexid) {
        P2("Grammar::lexicalEntry");

        procthunk *pt = lexical_entry->GetThunk();
        if(! pt) return false;
        
        mach->SetRP(nf_lexical_entry);

        mach->WriteLocal(0, PTR2c(word.GetAddress()));
        mach->WriteLocal(1, PTR2c(lexid.GetAddress()));
        
        pt->exec(*mach);

        if( mach->GetIP() == NULL ) return false;
	return mach->Execute(mach->GetIP());
    }
    
    bool Grammar::lexicalEntrySign(FSP lexid, FSP sign) {
        P2("Grammar::lexicalEntrySign");

        procthunk *pt = lexical_entry_sign->GetThunk();
        if(! pt) return false;
        
        mach->SetRP(nf_lexical_entry_sign);

        mach->WriteLocal(0, PTR2c(lexid.GetAddress()));
        mach->WriteLocal(1, PTR2c(sign.GetAddress()));
        
        pt->exec(*mach);

        if( mach->GetIP() == NULL ) return false;
	return mach->Execute(mach->GetIP());
    }
    
   bool Grammar::idSchemaUnary(FSP name, FSP dtr, FSP mother, FSP dcp)
    {
        P2("Grammar::idSchemaUnary");
        if ( ! id_schema_unary ) return false;
        procthunk *pt = id_schema_unary->GetThunk();
        if(! pt) return false;
    
        mach->SetRP(nf_id_schema_unary);

        mach->WriteLocal(0, PTR2c(name.GetAddress()));
        mach->WriteLocal(1, PTR2c(dtr.GetAddress()));
        mach->WriteLocal(2, PTR2c(mother.GetAddress()));
        mach->WriteLocal(3, PTR2c(dcp.GetAddress()));
    
        pt->exec(*mach);

        if( mach->GetIP() == NULL ) return false;
	return mach->Execute(mach->GetIP());
    }
    bool Grammar::idSchemaBinary(FSP name, FSP left, FSP right, FSP mother, FSP dcp)
    {
        P2("Grammar::idSchemaBinary");
        procthunk *pt = id_schema_binary->GetThunk();
        if(! pt) return false;
    
        mach->SetRP(nf_id_schema_binary);

        mach->WriteLocal(0, PTR2c(name.GetAddress()));
        mach->WriteLocal(1, PTR2c(left.GetAddress()));
        mach->WriteLocal(2, PTR2c(right.GetAddress()));
        mach->WriteLocal(3, PTR2c(mother.GetAddress()));
        mach->WriteLocal(4, PTR2c(dcp.GetAddress()));
    
        pt->exec(*mach);

        if( mach->GetIP() == NULL ) return false;
	return mach->Execute(mach->GetIP());
    }
     
    bool Grammar::reduceSign(FSP sign, FSP reduced_sign, FSP sign_plus) {
        P2("Grammar::reduceSign");

        procthunk *pt = reduce_sign->GetThunk();
        if(!pt) return false;
        
        mach->SetRP(nf_reduce_sign);

        mach->WriteLocal(0, PTR2c(sign.GetAddress()));
        mach->WriteLocal(1, PTR2c(reduced_sign.GetAddress()));
        mach->WriteLocal(2, PTR2c(sign_plus.GetAddress()));

        pt->exec(*mach);

        if( mach->GetIP() == NULL ) return false;
	return mach->Execute(mach->GetIP());
    }

    bool Grammar::rootSign(FSP sign) {
        P2("Grammar::rootSign");
      procthunk *pt = root_sign->GetThunk();
      if(! pt) return false;

      mach->SetRP(nf_root_sign);

      mach->WriteLocal(0, PTR2c(sign.GetAddress()));

      pt->exec(*mach);

      if( mach->GetIP() == NULL ) return false;
      return mach->Execute(mach->GetIP());
    }
    
} // end of namespace up

/*
 * $Log: Grammar.cc,v $
 * Revision 1.3  2005/06/21 10:17:12  yusuke
 *
 * copyright 񤭤ޤ
 *
 * Revision 1.2  2005/06/16 07:25:36  yusuke
 *
 * integrate-up 硥
 * version 2.0
 *
 * Revision 1.1.2.7  2005/06/11 04:05:02  yusuke
 *
 * cout -> *lilfes::output_stream
 * cerr -> *lilfes::error_stream
 *
 * Revision 1.1.2.6  2005/06/08 05:59:42  ninomi
 * P2ץեĤʤޤ
 *
 * Revision 1.1.2.5  2005/06/07 08:03:12  yusuke
 *
 * grammar_init -> parser_init
 *
 * Revision 1.1.2.4  2005/06/06 12:01:02  yusuke
 *
 * Ȥꤢ sample_hpsg  warning Фʤ褦ˤޤ
 *
 * Revision 1.1.2.3  2005/06/06 08:53:41  ninomi
 * IPTrailStackطľޤlilfespredicate callʤޤ
 * quick checkidҤˤΤnameǤҤ褦ˤޤ
 *
 * Revision 1.1.2.2  2005/06/05 13:14:21  ninomi
 * Ȥꤢ񴹤ޤǤФäϤ줫
 *
 * Revision 1.1.2.1  2005/06/05 10:12:18  ninomi
 * upmayz/srcβ˰ܤϤޤ
 *
 * Revision 1.8  2005/04/17 10:03:07  tsuruoka
 * add idSchemaBinary()
 *
 * Revision 1.7  2004/11/16 09:14:07  ninomi
 * Artistic LicenseƬˤĤޤ
 *
 * Revision 1.6  2004/07/02 06:40:41  ninomi
 * inittermޤΥХȤޤ
 *
 * Revision 1.5  2004/07/02 03:25:48  ninomi
 * QcFomΣĤΤơ
 * Parser->ParserBasic->ParserCky->ParserCkyQc->ParserFom褦ˤޤQuickCheckenableQuickCheck, disableQuickCheckOn/OffǤޤ
 *
 * Revision 1.4  2004/07/01 09:55:45  ninomi
 * FomManager::~FomManager()Grammar::~Grammar()lilfesҸfom_termgrammar_term򤽤줾ƤӽФ褦ˤޤ
 *
 * Revision 1.3  2004/07/01 09:24:09  ninomi
 * FomManager::init()Grammar::init()fom_initgrammar_init
 * lilfesҸƽФ褦ˤʤޤ
 *
 * Revision 1.2  2004/06/30 16:19:15  yusuke
 *
 * iterative parse 
 *
 * Revision 1.1  2004/06/30 07:50:29  ninomi
 * up 0.2release versionǤ
 *
 */
