/*
 * $Id: Grammar.h,v 1.3 2005/06/21 10:17:12 yusuke Exp $
 *
 *  Copyright (c) 2005, Tsujii Laboratory, The University of Tokyo.
 *  All rights reserved.
 *
 */

#ifndef __Grammar_h
#define __Grammar_h

#include <liblilfes/structur.h>
#include <liblilfes/proc.h>
#include <liblilfes/machine.h>

namespace up {
    
    class Grammar {
    private:
	bool is_open;
        
        int nf_sentence_to_word_lattice;
        int nf_lexical_entry;
        int nf_lexical_entry_sign;
        int nf_reduce_sign;
        int nf_root_sign;
        int nf_id_schema_unary;
        int nf_id_schema_binary;
    
        lilfes::procedure *sentence_to_word_lattice;
        lilfes::procedure *lexical_entry;
        lilfes::procedure *lexical_entry_sign;
        lilfes::procedure *reduce_sign;
        lilfes::procedure *root_sign;
        lilfes::procedure *id_schema_unary;
        lilfes::procedure *id_schema_binary;

        bool is_unary;
        
    protected:        
        lilfes::machine* mach;
    public:
        Grammar() : is_open(false) {}
        virtual ~Grammar() {
            term();
        }
        virtual bool isUnary() { return is_unary; }
        virtual bool init(lilfes::machine* m);
        virtual bool term();
        virtual bool sentence2WordLattice(lilfes::FSP sentence, lilfes::FSP wordlattice);
        virtual bool lexicalEntry(lilfes::FSP word, lilfes::FSP lexid);
        virtual bool lexicalEntrySign(lilfes::FSP lexid, lilfes::FSP sign);
        virtual bool idSchemaUnary(lilfes::FSP name, lilfes::FSP dtr, lilfes::FSP mother, lilfes::FSP dcp);
        virtual bool idSchemaBinary(lilfes::FSP name, lilfes::FSP l, lilfes::FSP r, lilfes::FSP m, lilfes::FSP dcp);
        virtual bool reduceSign(lilfes::FSP sign, lilfes::FSP reduced_sign, lilfes::FSP sign_plus);
        virtual bool rootSign(lilfes::FSP sign);
    };
}

#endif // __Grammar_h

/*
 * $Log: Grammar.h,v $
 * Revision 1.3  2005/06/21 10:17:12  yusuke
 *
 * copyright 񤭤ޤ
 *
 * Revision 1.2  2005/06/16 07:25:36  yusuke
 *
 * integrate-up 硥
 * version 2.0
 *
 * Revision 1.1.2.6  2005/06/07 14:22:05  yusuke
 *
 * forestmaker äư褦ˤʤä
 *
 * Revision 1.1.2.5  2005/06/07 08:03:12  yusuke
 *
 * grammar_init -> parser_init
 *
 * Revision 1.1.2.4  2005/06/06 08:53:41  ninomi
 * IPTrailStackطľޤlilfespredicate callʤޤ
 * quick checkidҤˤΤnameǤҤ褦ˤޤ
 *
 * Revision 1.1.2.3  2005/06/05 14:10:48  ninomi
 * ѥ뤬Ȥ褦ˤʤޤ
 *
 * Revision 1.1.2.2  2005/06/05 13:14:21  ninomi
 * Ȥꤢ񴹤ޤǤФäϤ줫
 *
 * Revision 1.1.2.1  2005/06/05 10:12:18  ninomi
 * upmayz/srcβ˰ܤϤޤ
 *
 * Revision 1.9  2005/04/17 10:02:39  tsuruoka
 * add idSchemaBinary()
 *
 * Revision 1.8  2004/11/16 09:13:54  ninomi
 * Artistic LicenseƬˤĤޤ
 *
 * Revision 1.7  2004/09/17 14:59:58  yusuke
 *
 * ҸƽФ virtual ˤ
 *
 * Revision 1.6  2004/07/02 06:40:35  ninomi
 * inittermޤΥХȤޤ
 *
 * Revision 1.5  2004/07/01 13:05:35  ninomi
 * FomManagerGrammaropenʾ֤ΤȤΤdestructorlilfes???term
 * ¹Ԥ褦ˤޤ
 *
 * Revision 1.4  2004/07/01 09:55:33  ninomi
 * FomManager::~FomManager()Grammar::~Grammar()lilfesҸfom_termgrammar_term򤽤줾ƤӽФ褦ˤޤ
 *
 * Revision 1.3  2004/07/01 09:23:49  ninomi
 * FomManager::init()Grammar::init()fom_initgrammar_init
 * lilfesҸƽФ褦ˤʤޤ
 *
 * Revision 1.2  2004/06/30 16:19:14  yusuke
 *
 * iterative parse 
 *
 * Revision 1.1  2004/06/30 07:50:09  ninomi
 * up 0.2release versionǤ
 *
 */
