/**********************************************************************
 *
 *  Copyright (c) 2005, Tsujii Laboratory, The University of Tokyo.
 *  All rights reserved.
 *
 *  @file LexExtractor.h
 *  @version Time-stamp: <2005-06-21 19:06:15 yusuke>
 *  Extracting lexical entries from derivations
 *
 **********************************************************************/

#ifndef MAYZ_LEX_EXTRACTOR_H

#define MAYZ_LEX_EXTRACTOR_H

#include "config_mayz.h"
#include <liblilfes/builtin.h>
#include "Derivation.h"
#include "Lexicon.h"
#include "TemplateTable.h"
#include "Lexbank.h"
#include "LexEntry.h"
#include "WordCounter.h"

namespace mayz {

  class LexExtractorException : public std::exception {
  private:
    std::string message;
  public:
    LexExtractorException( const char* m ) : message( m ) {}
    LexExtractorException( const std::string& m ) : message( m ) {}
    ~LexExtractorException() throw () {}
    const char* what() const throw () { return message.c_str(); }
  };

  class LexExtractorFailure : public std::exception {
  private:
    std::string message;
  public:
    LexExtractorFailure( const char* m ) : message( m ) {}
    LexExtractorFailure( const std::string& m ) : message( m ) {}
    ~LexExtractorFailure() throw () {}
    const char* what() const throw () { return message.c_str(); }
  };

  class LexExtractor {
  private:
    lilfes::machine* mach;
    Lexicon* lexicon;
    TemplateTable* temp_table;
    Lexbank* lexbank;
    WordCounter* word_count;
    FSArray< lilfes::FSP, lilfes::FSP > temp_reverse_table;
    int num_templates;
    int num_all_derivations;
    int num_success;
    int num_total_words;

    std::vector< lilfes::FSP > lex_entry_list;

  protected:
    void applyLexicalEntryTemplate( Derivation& terminal, lilfes::FSP lex_template );
    void applyInverseLexicalRule( Word word, lilfes::FSP lex_template, lilfes::FSP key, lilfes::FSP lexeme_template, lilfes::FSP lex_rule_list );
    void applyLexemeName( Word word, lilfes::FSP lex_template, std::string& name );
    void applyWordCountKey( lilfes::FSP key, lilfes::FSP word_key );

    //bool makeLexEntry( lilfes::FSP word, const std::string& name, lilfes::FSP lex_rule_list, LexEntry lex_entry );

    virtual void extractMain( int sentence_id, Derivation derivation );

  public:
    LexExtractor( lilfes::machine* m, Lexicon* l, TemplateTable* t, Lexbank* b, WordCounter* w )
      : mach( m ),
        lexicon( l ),
        temp_table( t ),
        lexbank( b ),
        word_count( w ),
        temp_reverse_table( m ),
        num_templates( 0 ),
        num_all_derivations( 0 ),
        num_success( 0 ),
        num_total_words( 0 ) {
    }
    virtual ~LexExtractor() {}

    virtual void initialize();
    virtual void extract( int sentence_id, Derivation derivation );

    virtual void showStatistics( std::ostream& os, const std::string& indent = "" ) const;
  };

}

#endif // MAYZ_LEX_EXTRACTOR_H
