/**********************************************************************
 *
 *  Copyright (c) 2005, Tsujii Laboratory, The University of Tokyo.
 *  All rights reserved.
 *
 *  @file Lexbank.h
 *  @version Time-stamp: <2005-06-21 19:06:03 yusuke>
 *  Implementation of lexbank (a set of lists of lexical entries)
 *
 **********************************************************************/

#ifndef MAYZ_LEXBANK_H

#define MAYZ_LEXBANK_H

#include "config_mayz.h"
#include "FSArray.h"
#include <liblilfes/builtin.h>
#include <vector>

namespace mayz {

  class LexbankException : public std::exception {
  private:
    std::string message;
  public:
    LexbankException( const char* m ) : message( m ) {}
    LexbankException( const std::string& m ) : message( m ) {}
    ~LexbankException() throw () {}
    const char* what() const throw () { return message.c_str(); }
  };

  class Lexbank {
  private:
    lilfes::machine* mach;
    FSArray< int, std::vector< lilfes::FSP > > db;
    size_t num_sentences;
    size_t max_id;

  public:
    typedef FSArray< int, std::vector< lilfes::FSP > >::iterator iterator;
    iterator begin() { return db.begin(); }
    iterator end() { return db.end(); }

  public:
    Lexbank( lilfes::machine* m ) : mach( m ), db( m ), num_sentences( 0 ), max_id( 0 ) {
    }
    virtual ~Lexbank() {}

    size_t numSentences() const { return num_sentences; }
    size_t maxID() const { return max_id; }

    bool insert( size_t i, const std::vector< lilfes::FSP >& lexentries ) {
      if ( i > max_id ) max_id = i;
      ++num_sentences;
      return db.insert( i, lexentries );
    }
    bool find( size_t i, std::vector< lilfes::FSP >& lexentries ) {
      return db.find( i, lexentries );
    }

    virtual bool load( const std::string& filename );
    virtual bool save( const std::string& filename );
  };

}

#endif // MAYZ_LEXBANK_H
