/**********************************************************************
 *
 *  Copyright (c) 2005, Tsujii Laboratory, The University of Tokyo.
 *  All rights reserved.
 *
 *  @file Lexicon.h
 *  @version Time-stamp: <2005-06-21 19:05:53 yusuke>
 *  Implementation of a lexicon (mappings from words to templates)
 *
 **********************************************************************/

#ifndef MAYZ_LEXICON_H

#define MAYZ_LEXICON_H

#include "config_mayz.h"
#include "FSArray.h"
#include "LexEntry.h"
#include <liblilfes/builtin.h>
#include <string>
#include <vector>

namespace mayz {

  class Lexicon {
  private:
    lilfes::machine* mach;
    FSArray< lilfes::FSP, std::vector< lilfes::FSP > > lexicon_db;
    //FSArray< lilfes::FSP, int > count_db;

  public:
    typedef FSArray< lilfes::FSP, std::vector< lilfes::FSP > >::iterator iterator;
    iterator begin() { return lexicon_db.begin(); }
    iterator end() { return lexicon_db.end(); }

  public:
    Lexicon( lilfes::machine* m ) : mach( m ), lexicon_db( mach ) {
    }
    virtual ~Lexicon() {}

    virtual bool add( lilfes::FSP word, const LexTemplate& name );
    virtual bool insert( lilfes::FSP word, const std::vector< lilfes::FSP >& temp_list ) {
      return lexicon_db.insert( word, temp_list );
    }
    virtual bool lookup( lilfes::FSP word, std::vector< lilfes::FSP >& temp_list ) {
      return lexicon_db.find( word, temp_list );
    }
    virtual bool erase( lilfes::FSP word ) {
      return lexicon_db.erase( word );
    }
//     virtual bool incCount( lilfes::FSP word, int c = 1 ) {
//       int word_count = 0;
//       if ( ! count_db.find( word, word_count ) ) {
//         word_count = 0;
//       }
//       word_count += c;
//       if ( ! count_db.insert( word, word_count ) ) {
//         return false;
//       }
//       return true;
//     }
//     virtual bool getCount( lilfes::FSP word, int& count ) {
//       return count_db.find( word, count );
//     }

    virtual bool load( const std::string& lexfile );
    virtual bool save( const std::string& lexfile );
  };

}

#endif // MAYZ_LEXICON_H
