/*
 * $Id: ParserBuiltin.cc,v 1.5 2005/07/29 03:03:45 yusuke Exp $
 *
 *  Copyright (c) 2005, Tsujii Laboratory, The University of Tokyo.
 *  All rights reserved.
 *
 */

///
/// built-in predicates
///

#include <liblilfes/builtin.h>
#include "Fsp.h"
#include "Extent.h"
#include "Parser.h"
#include "ParserCkyFom.h"
#include "ParserBuiltin.h"

static const char* PARSER_MODULE = "mayz:parser";

namespace up {
    Parser* Builtin::parser = NULL;
}

namespace lilfes {

    bool up_set_grammar_name(machine&, FSP arg1)
    {
        if ( ! arg1.IsString() ) {
          RUNWARN( "1st argument of set_grammar_name/1 must be a string" );
          return false;
        }
        std::string gn = arg1.ReadString();
        if ( ! up::Builtin::getParser() ) {
          RUNWARN( "set_grammar_name/1: parser is not initialized yet" );
          return false;
        }
        up::Builtin::getParser()->setGrammarName(gn);
        return true;
    }
    bool up_get_grammar_name(machine& m, FSP arg1)
    {
        if ( ! up::Builtin::getParser() ) {
          RUNWARN( "get_grammar_name/1: parser is not initialized yet" );
          return false;
        }
        std::string& gn = up::Builtin::getParser()->getGrammarName();
        return arg1.Unify(FSP(m, gn.c_str()));
    }
    bool up_set_grammar_version(machine&, FSP arg1)
    {
        if ( ! arg1.IsString() ) {
          RUNWARN( "1st argument of set_grammar_version/1 must be a string" );
          return false;
        }
        std::string gn = arg1.ReadString();
        if ( ! up::Builtin::getParser() ) {
          RUNWARN( "set_grammar_version/1: parser is not initialized yet" );
          return false;
        }
        up::Builtin::getParser()->setGrammarVersion(gn);
        return true;
    }
    bool up_get_grammar_version(machine& m, FSP arg1)
    {
        if ( ! up::Builtin::getParser() ) {
          RUNWARN( "get_grammar_version/1: parser is not initialized yet" );
          return false;
        }
        std::string& gn = up::Builtin::getParser()->getGrammarVersion();
        return arg1.Unify(FSP(m, gn.c_str()));
    }
    bool up_set_limit_sentence_length(machine&, FSP arg1)
    {
        if( ! arg1.IsInteger() ) { RUNWARN("1st argument of set_limit_sentence_length must be an integer"); return false; }
        if ( ! up::Builtin::getParser() ) {
          RUNWARN( "set_limit_sentence_length/1: parser is not initialized yet" );
          return false;
        }
        up::Builtin::getParser()->setLimitWords(arg1.ReadInteger());
        return true;
        
    }
    bool up_get_limit_sentence_length(machine& m, FSP arg1)
    {
        if ( ! up::Builtin::getParser() ) {
          RUNWARN( "get_limit_sentence_length/1: parser is not initialized yet" );
          return false;
        }
        return arg1.Unify(FSP(m, up::Builtin::getParser()->getLimitWords()));
    }
    bool up_set_limit_edge_number(machine&, FSP arg1)
    {
        if( ! arg1.IsInteger() ) { RUNWARN("1st argument of set_limit_edge_number must be an integer"); return false; }
        if ( ! up::Builtin::getParser() ) {
          RUNWARN( "set_limit_edge_number/1: parser is not initialized yet" );
          return false;
        }
        up::Builtin::getParser()->setLimitEdges(arg1.ReadInteger());
        return true;
        
    }
    bool up_get_limit_edge_number(machine& m, FSP arg1)
    {
        if ( ! up::Builtin::getParser() ) {
          RUNWARN( "get_limit_edge_number/1: parser is not initialized yet" );
          return false;
        }
        return arg1.Unify(FSP(m, up::Builtin::getParser()->getLimitEdges()));
    }

    bool up_get_parse_status( machine&, FSP arg1 )
    {
      if ( ! up::Builtin::getParser() ) {
        RUNWARN( "get_parse_status/1: parser is not initialized yet" );
        return false;
      }
      up::Parser::ParseStatus status = up::Builtin::getParser()->getParseStatus();
      module* parser_module = module::SearchModule( PARSER_MODULE );
      if ( ! parser_module ) {
        RUNWARN( "get_parse_status/1: parser module \"" << PARSER_MODULE << "\" not loaded yet" );
        return false;
      }
      type* ret = NULL;
      switch ( status ) {
      case up::Parser::SUCCESS:
        ret = parser_module->Search( "parse_success" );
        break;
      case up::Parser::PREPROC_ERROR:
        ret = parser_module->Search( "preprocess_error" );
        break;
      case up::Parser::TOO_LONG:
        ret = parser_module->Search( "parse_error_too_long" );
        break;
      case up::Parser::EDGE_LIMIT:
        ret = parser_module->Search( "parse_error_edge_limit" );
        break;
      default:
        ;
      }
      if ( ret == NULL ) return false;
      return arg1.Coerce( ret );
    }

    bool up_edge(machine& m, FSP arg1, FSP arg2)
    {
        if ( ! arg1.IsInteger() ) { RUNWARN("1st argument of edge_sign must be an integer"); return false; }

        if ( ! up::Builtin::getParser() ) {
          RUNWARN( "edge_sign/2: parser is not initialized yet" );
          return false;
        }
        up::Edge* e = up::Builtin::getParser()->getEdge((up::eserial) arg1.ReadInteger());
        if(! e) { RUNWARN("invalid serialno"); return false; }
        return arg2.Unify(FSP(m, e->getSign()));
    }
    bool up_edge_position(machine& m, FSP arg1, FSP arg2, FSP arg3)
    {
        if ( ! arg1.IsInteger() ) { RUNWARN( "1st argument of edge_position must be an integer" ); return false; }

        if ( ! up::Builtin::getParser() ) {
          RUNWARN( "edge_position/3: parser is not initialized yet" );
          return false;
        }
        up::Edge* e = up::Builtin::getParser()->getEdge((up::eserial) arg1.ReadInteger());
        if(! e) { RUNWARN("invalid serialno"); return false; }
        std::pair<int, int> position = e->getPosition();
        return arg2.Unify(FSP(m, position.first)) && arg3.Unify(FSP(m,position.second));
    }

    bool up_link_id_list(machine& m, FSP arg1, FSP arg2)
    {
        FSP f(m);
        FSP froot(m, f.GetAddress());

        if ( ! arg1.IsInteger() ) { RUNWARN("1st argument of edge_link_id_list must be an integer"); return false; }
    
        if ( ! up::Builtin::getParser() ) {
          RUNWARN( "edge_link_id_list/3: parser is not initialized yet" );
          return false;
        }
        up::Edge* e = up::Builtin::getParser()->getEdge((up::eserial) arg1.ReadInteger());
        if(! e) { RUNWARN("invalid serialno"); return false; }

        //up::FspMaker* fspm = up::Builtin::getFspMaker();

//         up::ParserCkyFom* parser = dynamic_cast< up::ParserCkyFom* >( up::Builtin::getParser() );
//         up::FomManager* fomm = NULL;
//         if ( parser != NULL ) { fomm = parser->getFomManager(); }
        std::vector<up::Link*>& iii = e->getLinks();
        std::vector<up::Link*>::iterator it = iii.begin(), last = iii.end();
        for(; it != last ; it++) {
//             FSP new_fsp = fspm->makeFSP(*it);
//             if ( fomm != NULL ) {
//                 FSP new_fsp2 = fspm->makeFSP( *it, fomm );
//                 new_fsp.Unify( new_fsp2 );
//             }
            FSP new_fsp( m, static_cast<mint>((*it)->getSerialNo()) );
            f.Follow(lilfes::hd).Unify(new_fsp);
            f = f.Follow(tl);
        }
        f.Coerce(nil);
        return arg2.Unify(froot);
    }

    bool up_link( machine&, FSP arg1, FSP arg2 )
    {
//         FSP f(m);
//         FSP froot(m, f.GetAddress());

        if ( ! arg1.IsInteger() ) { RUNWARN("1st argument of edge_link must be an integer"); return false; }

        if ( ! up::Builtin::getParser() ) {
          RUNWARN( "edge_link/2: parser is not initialized yet" );
          return false;
        }

        up::Link* link = up::Builtin::getParser()->getLink((up::lserial) arg1.ReadInteger());
        if(! link) { RUNWARN("invalid serialno"); return false; }

        up::FspMaker* fspm = up::Builtin::getParser()->getFspMaker();
        if(! fspm) { RUNWARN("FspMaker is not initialized"); return false; }

//         up::ParserCkyFom* parser = dynamic_cast< up::ParserCkyFom* >( up::Builtin::getParser() );
//         up::FomManager* fomm = NULL;
//         if ( parser != NULL ) { fomm = parser->getFomManager(); }
        FSP new_fsp = fspm->makeFSP(link);
//         if ( fomm != NULL ) {
//             FSP new_fsp2 = fspm->makeFSP( link, fomm );
//             new_fsp.Unify( new_fsp2 );
//         }
        return arg2.Unify(new_fsp);
  }

    bool up_get_word_lattice(machine& m, FSP arg1) {
        if ( ! up::Builtin::getParser() ) {
          RUNWARN( "get_word_lattice/1: parser is not initialized yet" );
          return false;
        }
        const up::WordLattice& wordlattice = up::Builtin::getParser()->getWordLattice();
        up::FspMaker* fspm = up::Builtin::getParser()->getFspMaker();
        if(! fspm) { RUNWARN("FspMaker is not initialized"); return false; }
    
        up::WordLattice::const_iterator it = wordlattice.begin(), last = wordlattice.end();
        FSP f(m);
        FSP froot(m, f.GetAddress());
        for(; it != last ; it++) {
            f.Follow(hd).Unify(fspm->makeFSP(*it));
            f = f.Follow(tl);
        }
        f.Coerce(nil);
        return arg1.Unify(froot);
    }

    bool up_get_sentence(machine& m, FSP arg1)
    {
        if ( ! up::Builtin::getParser() ) {
          RUNWARN( "get_sentence/1: parser is not initialized yet" );
          return false;
        }
        const std::string& sent = up::Builtin::getParser()->getSentence();
        return arg1.Unify(FSP(m, sent.c_str()));
    }

    bool up_get_sentence_length(machine& m, FSP arg1)
    {
        if ( ! up::Builtin::getParser() ) {
          RUNWARN( "get_sentence_length/1: parser is not initialized yet" );
          return false;
        }
        int n = up::Builtin::getParser()->getSentenceLength();
        return arg1.Unify(FSP(m, (mint) n));
    }

    bool up_get_edge_number(machine& m, FSP arg1)
    {
        if ( ! up::Builtin::getParser() ) {
          RUNWARN( "get_edge_number/1: parser is not initialized yet" );
          return false;
        }
        int n = up::Builtin::getParser()->getEdgeNumber();
        return arg1.Unify(FSP(m, (mint) n));
    }

    bool up_edge_id_list(machine& m, FSP arg1, FSP arg2, FSP arg3)
    {
        if(! (arg1.IsInteger() && arg2.IsInteger() ) ) { return false; }

        if ( ! up::Builtin::getParser() ) {
          RUNWARN( "edge_id_list/3: parser is not initialized yet" );
          return false;
        }

        int i = arg1.ReadInteger();
        int j = arg2.ReadInteger();
        FSP f(m);
        FSP froot(m, f.GetAddress());

        std::vector<up::Edge*> &ee = up::Builtin::getParser()->getEdges(i, j);
        std::vector<up::Edge*>::iterator it = ee.begin(), last = ee.end();

        for(; it != last ; it++) {
            mint eSN = (*it)->getSerialNo();
            f.Follow(hd).Unify(FSP(m, eSN));
            f = f.Follow(tl);
        }
        f.Coerce(nil);
    
        return arg3.Unify(froot);
    }

    bool up_parse(machine&, FSP arg1)
    {
        if(! arg1.IsString() ) { return false; }
        std::string sentence = arg1.ReadString();

        if ( ! up::Builtin::getParser() ) {
          RUNWARN( "parse/1: parser is not initialized yet" );
          return false;
        }

        if(! up::Builtin::getParser()->parse(sentence)) return false;
        return true;
    }
//     bool up_parse_with_bracket(machine&, FSP arg1)
//     {
//         if(! arg1.IsString() ) { return false; }
//         std::string sentence = arg1.ReadString();
//         if(! up::Builtin::getParser()->parseWithBracket(sentence)) return false;
//         return true;
//     }
    bool up_parse_word_lattice(machine&, FSP arg1)
    {
        if ( ! up::Builtin::getParser() ) {
          RUNWARN( "parse_word_lattice/1: parser is not initialized yet" );
          return false;
        }

        up::WordLattice wordlattice;
        up::FspMaker* fspm = up::Builtin::getParser()->getFspMaker();
        if(!fspm) { RUNWARN("FspMaker is not initialized"); return false; }
        while( arg1.GetType()->IsSubType(cons) ) {
            wordlattice.push_back(fspm->newExtent(arg1.Follow(hd)));
            arg1 = arg1.Follow(tl);
        }
        if(! up::Builtin::getParser()->parseWordLattice(wordlattice)) return false;
        return true;
    }

    bool pcky_set_beam_thresholding_params(machine&, FSP arg1, FSP arg2)
    {
        if ( ! arg1.IsInteger() )
        { RUNWARN("1st argument of pcky_set_thresholding_params must be an integer"); return false; }
        if ( ! arg2.IsFloat() )
        { RUNWARN("2nd argument of pcky_set_thresholding_params must be a float"); return false; }

        up::Parser* up = up::Builtin::getParser();
        up::ParserCkyFom* pcup = dynamic_cast<up::ParserCkyFom*>(up);
        if(! pcup ) { RUNWARN("parser is not set, or parser is not ParserCkyFom"); return false; }

        pcup->setBeamThresholdNum(arg1.ReadInteger());
        pcup->setBeamThresholdWidth(arg2.ReadFloat());

        return true;
    }

    bool pcky_get_edge_fom(machine& m, FSP arg1, FSP arg2)
    {
        if ( ! arg1.IsInteger() ) { RUNWARN("1st argument of edge_fom must be an integer"); return false; }
        up::eserial sn = (up::eserial) arg1.ReadInteger();
        up::Parser* up = up::Builtin::getParser();
        up::ParserCkyFom* pcup = dynamic_cast<up::ParserCkyFom*>(up);
        if(! pcup ) { RUNWARN("parser is not set, or parser is not ParserCkyFom"); return false; }
        return arg2.Unify(FSP(m, (mfloat) pcup->getEdgeFom(sn)));
    }

    bool pcky_get_edge_link_fom(machine& m, FSP arg1, FSP arg2)
    {
        if ( ! arg1.IsInteger() ) { RUNWARN("1st argument of edge_link_fom must be an integer"); return false; }
        up::lserial sn = (up::lserial) arg1.ReadInteger();
        up::Parser* up = up::Builtin::getParser();
        up::ParserCkyFom* pcup = dynamic_cast<up::ParserCkyFom*>(up);
        if(! pcup ) { RUNWARN("parser is not set, or parser is not ParserCkyFom"); return false; }
        return arg2.Unify(FSP(m, (mfloat) pcup->getLinkFom(sn)));
    }

    bool up_set_parser_mode(machine&, FSP arg1, FSP arg2)
    {
        if(! arg1.IsString() ) { RUNWARN("1st argument of set_parser_mode must be a string"); return false; }
        std::string attr = arg1.ReadString();
        if(attr == "quick check path") {
            up::ParserCky* parser = dynamic_cast< up::ParserCky* >( up::Builtin::getParser() );
            if(! parser) return false;
            parser->setQuickCheckPath(arg2);
            return true;
        } else if(attr == "enable quick check") {
            up::ParserCky* parser = dynamic_cast< up::ParserCky* >( up::Builtin::getParser() );
            if(! parser) return false;
            if( arg2.IsInteger() && arg2.ReadInteger() == 0 ) {
              parser->disableQuickCheck();
            } else {
              if(! parser->enableQuickCheck() ) return false;
            }
            return true;
        }
        
        RUNWARN("set_parser_mode/2: no matching attribute for setting parser mode: " << attr );
        return false;
    }
    bool up_get_parser_mode(machine& m, FSP arg1, FSP arg2)
    {
        if(! arg1.IsString() ) { RUNWARN("1st argument of get_parser_mode must be a string"); return false; }
        std::string attr = arg1.ReadString();
        if(attr == "quick check path") {
            up::ParserCky* parser = dynamic_cast< up::ParserCky* >( up::Builtin::getParser() );
            if(! parser) return false;
            const up::Fs* path = parser->getQuickCheckPath();
            if(! path) return false;
            return arg2.Unify( FSP( m, path ) );
        } else if(attr == "enable quick check") {
            up::ParserCky* parser = dynamic_cast< up::ParserCky* >( up::Builtin::getParser() );
            if(! parser) return false;
            if(parser->doQuickCheck()) {
              return arg2.Unify(1);
            } else {
              return arg2.Unify(0);
            }
        }
        
        RUNWARN("get_parser_mode/2: no matching attribute for setting parser mode: " << attr );
        return false;
    }

    bool up_get_parser_name(machine& m, FSP arg1)
    {
        if ( ! up::Builtin::getParser() ) {
          RUNWARN( "get_parser_name/1: parser is not initialized yet" );
          return false;
        }

        std::string& gn = up::Builtin::getParser()->getParserName();
        return arg1.Unify(FSP(m, gn.c_str()));
    }

    bool up_get_parser_version(machine& m, FSP arg1)
    {
        if ( ! up::Builtin::getParser() ) {
          RUNWARN( "get_parser_version/1: parser is not initialized yet" );
          return false;
        }

        std::string& gn = up::Builtin::getParser()->getParserVersion();
        return arg1.Unify(FSP(m, gn.c_str()));
    }

    bool up_get_preprocessing_time(machine& m, FSP arg1)
    {
        if ( ! up::Builtin::getParser() ) {
          RUNWARN( "get_preprocessing_time/1: parser is not initialized yet" );
          return false;
        }
        int t = up::Builtin::getParser()->getPreprocessingTime();
        return arg1.Unify(FSP(m, (mint)t));
    }

    bool up_get_lexicon_lookup_time(machine& m, FSP arg1)
    {
        if ( ! up::Builtin::getParser() ) {
          RUNWARN( "get_lexicon_lookup_time/1: parser is not initialized yet" );
          return false;
        }
        int t = up::Builtin::getParser()->getLexiconLookupTime();
        return arg1.Unify(FSP(m, (mint)t));
    }

    bool up_get_parsing_time(machine& m, FSP arg1)
    {
        if ( ! up::Builtin::getParser() ) {
          RUNWARN( "get_parsing_time/1: parser is not initialized yet" );
          return false;
        }
        int t = up::Builtin::getParser()->getParsingTime();
        return arg1.Unify(FSP(m, (mint)t));
    }

    LILFES_BUILTIN_PRED_1(up_set_grammar_name, up_set_grammar_name);
    LILFES_BUILTIN_PRED_1(up_get_grammar_name, up_get_grammar_name);
    LILFES_BUILTIN_PRED_1(up_set_grammar_version, up_set_grammar_version);
    LILFES_BUILTIN_PRED_1(up_get_grammar_version, up_get_grammar_version);
    LILFES_BUILTIN_PRED_1(up_set_limit_sentence_length, up_set_limit_sentence_length);
    LILFES_BUILTIN_PRED_1(up_get_limit_sentence_length, up_get_limit_sentence_length);
    LILFES_BUILTIN_PRED_1(up_set_limit_edge_number, up_set_limit_edge_number);
    LILFES_BUILTIN_PRED_1(up_get_limit_edge_number, up_get_limit_edge_number);
    LILFES_BUILTIN_PRED_1(up_get_parse_status, up_get_parse_status);
    LILFES_BUILTIN_PRED_2(up_edge, up_edge);
    LILFES_BUILTIN_PRED_3(up_edge_position, up_edge_position);
    LILFES_BUILTIN_PRED_2(up_link_id_list, up_link_id_list);
    LILFES_BUILTIN_PRED_2(up_link, up_link);
    LILFES_BUILTIN_PRED_1(up_get_sentence, up_get_sentence);
    LILFES_BUILTIN_PRED_1(up_get_word_lattice, up_get_word_lattice);
    LILFES_BUILTIN_PRED_1(up_get_sentence_length, up_get_sentence_length);
    LILFES_BUILTIN_PRED_1(up_get_edge_number, up_get_edge_number);
    LILFES_BUILTIN_PRED_3(up_edge_id_list, up_edge_id_list);
    LILFES_BUILTIN_PRED_1(up_parse, up_parse);
//     LILFES_BUILTIN_PRED_1(up_parse_with_bracket, up_parse_with_bracket);
    LILFES_BUILTIN_PRED_1(up_parse_word_lattice, up_parse_word_lattice);
    LILFES_BUILTIN_PRED_2(pcky_set_beam_thresholding_params, pcky_set_beam_thresholding_params);
    LILFES_BUILTIN_PRED_2(pcky_get_edge_fom, pcky_get_edge_fom);
    LILFES_BUILTIN_PRED_2(pcky_get_edge_link_fom, pcky_get_edge_link_fom);
    LILFES_BUILTIN_PRED_2(up_set_parser_mode, up_set_parser_mode);
    LILFES_BUILTIN_PRED_2(up_get_parser_mode, up_get_parser_mode);
    LILFES_BUILTIN_PRED_1(up_get_parser_name, up_get_parser_name);
    LILFES_BUILTIN_PRED_1(up_get_parser_version, up_get_parser_version);
    LILFES_BUILTIN_PRED_1(up_get_preprocessing_time, up_get_preprocessing_time);
    LILFES_BUILTIN_PRED_1(up_get_lexicon_lookup_time, up_get_lexicon_lookup_time);
    LILFES_BUILTIN_PRED_1(up_get_parsing_time, up_get_parsing_time);
}
/*
 * $Log: ParserBuiltin.cc,v $
 * Revision 1.5  2005/07/29 03:03:45  yusuke
 *
 * preproc_error -> preprocess_error
 *
 * Revision 1.4  2005/06/22 08:31:14  yusuke
 *
 * ѡ builtin ƤФ줿 fail 褦ˤ
 *
 * Revision 1.3  2005/06/21 10:17:12  yusuke
 *
 * copyright 񤭤ޤ
 *
 * Revision 1.2  2005/06/16 07:25:36  yusuke
 *
 * integrate-up 硥
 * version 2.0
 *
 * Revision 1.1.2.13  2005/06/08 05:23:28  yusuke
 *
 * quick check ط
 *
 * Revision 1.1.2.12  2005/06/08 04:29:09  yusuke
 *
 * get_parser_mode/2 
 *
 * Revision 1.1.2.11  2005/06/08 03:30:43  yusuke
 *
 * parse_with_brackets ѻߡ
 * get_parse_status 
 *
 * Revision 1.1.2.10  2005/06/07 07:37:28  ninomi
 * Parserݥ饹ľޤ
 *
 * Revision 1.1.2.9  2005/06/06 15:31:20  yusuke
 *
 * forestmaker, mayzup ѥǤ褦ˤ
 * ޤ FMGrammar ȤʤΤǡforestmaker ưʤ
 *
 * Revision 1.1.2.8  2005/06/06 15:00:34  yusuke
 *
 * unused variable  warning ä
 *
 * Revision 1.1.2.7  2005/06/06 14:56:05  yusuke
 *
 * get_parser_mode, get_parse_status ʳϼ
 *
 * Revision 1.1.2.6  2005/06/06 13:23:46  yusuke
 *
 * edge_link_id_list ư褦ˤʤä
 * edge_link Ϥޤ
 *
 * Revision 1.1.2.5  2005/06/06 10:39:54  ninomi
 * quick check pathꡢʬ񤭤ޤ
 *
 * Revision 1.1.2.4  2005/06/06 09:44:28  ninomi
 * builtinޤդ䤷ޤ
 *
 * Revision 1.1.2.3  2005/06/05 14:10:48  ninomi
 * ѥ뤬Ȥ褦ˤʤޤ
 *
 * Revision 1.1.2.2  2005/06/05 13:14:21  ninomi
 * Ȥꤢ񴹤ޤǤФäϤ줫
 *
 * Revision 1.1.2.1  2005/06/05 10:12:19  ninomi
 * upmayz/srcβ˰ܤϤޤ
 *
 * Revision 1.8  2005/02/26 12:18:16  yusuke
 *
 * edge_info γΨȡfom_equivalent_reduction 裳¸ơ
 * edge_info_fom 褦ˤ
 *
 * Revision 1.7  2004/06/30 07:42:12  ninomi
 * up 0.2˰ܹԤ뤿0.1θŤե
 *
 * Revision 1.6  2004/05/21 07:50:53  yusuke
 *
 * edge_fom/2
 *
 * Revision 1.5  2004/05/20 23:21:00  ninomi
 * cup, pcup, qcup, pqcupСǤcup, qcupưƤޤ
 *
 * Revision 1.4  2004/05/18 16:40:19  ninomi
 * PCKYäޤ
 *
 * Revision 1.3  2004/05/16 13:44:47  ninomi
 * up_pasreup_parse_with_bracketup_parse_word_lattice˺Ƥ
 * Τɲäޤ
 *
 * Revision 1.2  2004/05/15 17:49:55  ninomi
 * *** empty log message ***
 *
 * Revision 1.1  2004/05/15 14:07:53  ninomi
 * BuiltinѤΥ饹Ĥޤ
 *
 * Revision 1.1  2004/05/15 13:14:17  ninomi
 * BuiltinҸ򤤤եĤޤ
 *
 * Revision 1.1  2004/05/15 07:59:23  ninomi
 * mainLiLFeSΥ󥿡եäޤ
 *
 */
