/*
 * $Id: ParserCky.cc,v 1.4 2005/07/27 09:00:43 yusuke Exp $
 *
 *  Copyright (c) 2005, Tsujii Laboratory, The University of Tokyo.
 *  All rights reserved.
 *
 */

#include "ParserCky.h"

namespace up {
    using namespace lilfes;

    bool ParserCky::parseLex(const WordLattice& wordlattice) {
        P2("ParserCky::parseLex");
        WordLattice::const_iterator it = wordlattice.begin(), last = wordlattice.end();
        for(; it != last ; it++) {
	    ExtentWord *exw = dynamic_cast<ExtentWord*>(*it);
            if(!exw) continue;
            
            int i = exw->getLeftPosition();
            int j = exw->getRightPosition();
            FSP word(mach, exw->getWord());
        
            static std::vector<Edge*> queue;
	    static std::vector<Fs*> signs;
	    static std::vector<Link*> links;

	    queue.clear();
	    signs.clear();
	    links.clear();

	    lookupLexicalEntry(i, j, word, signs, links);
            
            if(! storeEdge(i, j, signs, links, queue) ) return false;
            if( grammar->isUnary() )
                parseUnary(i, j, queue);
        }
        return true;
    }

    bool ParserCky::parsePhrase() {
        P2("ParserCky::parseIJ");

        int n = getSentenceLength();

        for( int j = 1; j <= n ; j++ )
            for( int i = j - 1 ; i >= 0 ; i-- ) {

                if(! chart->isAlive(i, j) ) continue;

                for( int k = i + 1; k < j ; k++ )
                    if(! parseK(i, k, j)) return false;

                if( grammar->isUnary() ) {
                  std::vector<Edge*> queue = chart->getEdges(i, j);
                  parseUnary(i, j, queue);
                }

            }
        return true;
    }

    bool ParserCky::storeEdge(int i, int j,
			      std::vector<Fs*>& signs,
			      std::vector<Link*>& links,
			      std::vector<Edge*> &queue) {
        P2("ParserCky::StoreEdge");
        ASSERT(0 <= i);
        ASSERT(i < j);
        ASSERT(j < (limit_words + 1));

        std::vector<Link*>::iterator it = links.begin(), last = links.end();
	std::vector<Fs*>::iterator it_sign = signs.begin();
    
        for(; it != last ; ++it, ++it_sign) {

            Fs* x = *it_sign;
        
            std::vector<Edge*> &xxx = chart->getEdges(i, j);
            std::vector<Edge*>::iterator cell_it = xxx.begin(), cell_last = xxx.end();

            bool factored = false;
                // if factored, store edge_info
            for(; cell_it != cell_last ; ++cell_it) {
                Edge* e = (*cell_it);
                Fs* y = e->getSign();
                if ( (*x) == (*y) ) {
                    e->addLink(*it);
		    eserial esn = e->getSerialNo();
                    (*it)->setMyEdgeSerialNo(esn);
                    delete x;
                    factored = true;
                    break;
                }
            }

                // not factored
            if (! factored ) {

                if( ((int) getEdgeManager()->getSerialCount()) >= getLimitEdges() ) { delete x; return false; }

                Edge *e = edgem->makeEdge(std::pair<int, int>(i, j), x);
		eserial esn = e->getSerialNo();
                
                e->addLink(*it);
                (*it)->setMyEdgeSerialNo(esn);
                
                chart->getEdges(i, j).push_back(e);
                queue.push_back(e);
            }
        }
        return true;
    }

    void ParserCky::lookupLexicalEntryCont(int i, int j, FSP, FSP lexid, FSP sign, FSP sign_plus,
					   std::vector<Fs*>& signs, std::vector<Link*>& links) {
        P2("ParserCky::lookupLexicalEntryCont");
	signs.push_back(sign.Serialize());
	links.push_back(linkm->makeLinkTerminal(std::make_pair(i, j),
                                                sign_plus.Serialize(),
                                                lexid.Serialize()));
    }
    
    void ParserCky::lookupLexicalEntry(int i, int j, FSP word,
				       std::vector<Fs*>& signs, std::vector<Link*>& links) {
        P2("ParserCky::lookupLexicalEntry");
        
	IPTrailStack iptrail(mach);
	FSP lexid(mach); FSP sign(mach); FSP reduced_sign(mach); FSP sign_plus(mach);

	bool loop = grammar->lexicalEntry(word, lexid);
        for(; loop ; loop = mach->NextAnswer() ) {
            //IPTrailStack iptrail2(mach);
            bool loop2 = grammar->lexicalEntrySign(lexid, sign);
            if ( ! loop2 ) continue;
            //for(; loop2 ; loop2 = mach->NextAnswer() ) {
            //IPTrailStack iptrail3(mach);
            bool loop3 = grammar->reduceSign(sign, reduced_sign, sign_plus);
            if ( ! loop3 ) continue;
            //for(; loop3 ; loop3 = mach->NextAnswer() ) {
            lookupLexicalEntryCont(i, j, word, lexid, reduced_sign, sign_plus, signs, links);
	}
    }
        
    void ParserCky::applyIdSchemaUnaryCont(Edge* e, FSP name, FSP, FSP mother, FSP sign_plus,
					   std::vector<Fs*>& signs, std::vector<Link*>& links) {
        P2("ParserCky::applyIdSchemaUnaryCont");
	signs.push_back(mother.Serialize());
	links.push_back(linkm->makeLinkUnary(std::make_pair(e->getLeftPosition(),
                                                            e->getRightPosition()),
                                             sign_plus.Serialize(),
                                             e,
                                             name.Serialize()));
    }
    void ParserCky::applyIdSchemaUnary(Edge* e, std::vector<Fs*>& signs, std::vector<Link*>& links) {
        P2("ParserCky::applyIdSchemaUnary");

        IPTrailStack iptrail(mach);
	FSP name(mach); FSP dtr(mach, e->getSign()); FSP mother(mach); FSP dcp(mach);
        FSP reduced_mother(mach); FSP sign_plus(mach);
        
	bool loop = grammar->idSchemaUnary(name, dtr, mother, dcp);
	for(; loop ; loop = mach->NextAnswer() ) {
            IPTrailStack iptrail2(mach);
            if(! builtin::lilfes_call(*mach, dcp) || mach->GetIP() == NULL ) continue;
            bool loop2 = mach->Execute(mach->GetIP());
            for(; loop2 ; loop2 = mach->NextAnswer() ) {
                //IPTrailStack iptrail3(mach);
                bool loop3 = grammar->reduceSign(mother, reduced_mother, sign_plus);
                if ( ! loop3 ) continue;
                //for(; loop3 ; loop3 = mach->NextAnswer() ) {
                applyIdSchemaUnaryCont(e, name, dtr, reduced_mother, sign_plus, signs, links);
                //}
            }
        }
    }
        
    bool ParserCky::parseUnary(int i, int j, std::vector<Edge*> &queue) {
        P2("ParserCky::parseUnary");
    
        while(! queue.empty() ) {

          //std::cerr << i << "," << j << "  " << queue.size() << std::endl;
	    
	    static std::vector<Fs*> signs;
	    static std::vector<Link*> links;
	    signs.clear();
	    links.clear();

            Edge* dtr_edge = queue.back();
            queue.pop_back();
	    
	    applyIdSchemaUnary(dtr_edge, signs, links);
            storeEdge(i, j, signs, links, queue);
        }
        return true;
    }


    void ParserCky::applyIdSchemaBinaryCont(Edge* x, Edge* y,
					    FSP name, FSP, FSP, FSP mother, FSP sign_plus,
					    std::vector<Fs*>& signs, std::vector<Link*>& links) {
        P2("ParserCky::applyIdSchemaBinaryCont");
	signs.push_back(mother.Serialize());
	links.push_back(linkm->makeLinkBinary(std::make_pair(x->getLeftPosition(),
								  y->getRightPosition()),
						   sign_plus.Serialize(),
						   x, y, name.Serialize()));
    }

    void ParserCky::applyIdSchemaBinary(Edge* x, Edge* y, FSP xfs, FSP yfs,
					  std::vector<Fs*>& signs, std::vector<Link*>& links) {
        P2("ParserCky::applyIdSchemaBinary");
        
        if(!doQuickCheck()) {

            IPTrailStack iptrail(mach);
            FSP name(mach); FSP mother(mach); FSP dcp(mach);
            FSP reduced_mother(mach); FSP sign_plus(mach);
            
            bool loop = grammar->idSchemaBinary(name, xfs, yfs, mother, dcp);
            for( ; loop ; loop = mach->NextAnswer() ) {
                IPTrailStack iptrail2(mach);
                if(! builtin::lilfes_call(*mach, dcp) || mach->GetIP() == NULL ) continue;
                bool loop2 = mach->Execute(mach->GetIP());
                for( ; loop2 ; loop2 = mach->NextAnswer() ) {
                    //IPTrailStack iptrail3(mach);
                    bool loop3 = grammar->reduceSign(mother, reduced_mother, sign_plus);
                    if ( ! loop3 ) continue;
                    //for(; loop3 ; loop3 = mach->NextAnswer() ) {
                    applyIdSchemaBinaryCont(x, y, name, xfs, yfs, reduced_mother, sign_plus, signs, links);
                    //}
                }
            }
        } else {
            
            eserial xsn = x->getSerialNo();
            eserial ysn = y->getSerialNo();

            if(!qc->isValid(xsn)) {
                qc->makeQca(xsn, x->getSign());
            }
        
            if(qc->getIdSchemaQca(xsn).size() > 0 && !qc->isValid(ysn)) {
                qc->makeQca(ysn, y->getSign());
            }
        
            std::vector<std::pair<Fs*, qc::QCA*> >&
                idschema_qca = qc->getIdSchemaQca(xsn);

            std::vector<std::pair<Fs*, qc::QCA*> >::iterator
                it = idschema_qca.begin(), last = idschema_qca.end();

            for(; it != last ; it++) {

                qc::QCA* qcx = it->second;
                qc::QCA* qcy = qc->getEdgeQca(ysn);
                if(! qc->quickCheck(qcx, qcy)) continue;

                IPTrailStack iptrail(mach);
                FSP name(mach, it->first); FSP mother(mach); FSP dcp(mach);
                FSP reduced_mother(mach); FSP sign_plus(mach);

                {
                    P2("applyIdSchemaBinary -- grammar->idSchemaBinary");
                    bool loop = grammar->idSchemaBinary(name, xfs, yfs, mother, dcp);
                    for(; loop ; loop = mach->NextAnswer() ) {
                        IPTrailStack iptrail2(mach);
                        {
                            P2("applyIdSchemaBianry -- dcp call");
                            if(! builtin::lilfes_call(*mach, dcp) || mach->GetIP() == NULL ) continue;
                            bool loop2 = mach->Execute(mach->GetIP());
                            for(; loop2 ; loop2 = mach->NextAnswer() ) {
                                //IPTrailStack iptrail3(mach);
                                {
                                    P2("applyIdSchemaBinary -- reduceSign");
                                    bool loop3 = grammar->reduceSign(mother, reduced_mother, sign_plus);
                                    if ( ! loop3 ) continue;
                                    //for(; loop3 ; loop3 = mach->NextAnswer() ) {
                                    applyIdSchemaBinaryCont(x, y, name, xfs, yfs, reduced_mother, sign_plus, signs, links);
                                    //}
                                }
                            }
                        }
                    }
                }
            }
        }
    }
    
    bool ParserCky::parseXY(int i, int j, Edge* x, FSP xfs, Edge*y, FSP yfs) {
	P2("ParserCky::parseXY");

	static std::vector<Edge*> queue;
	static std::vector<Fs*> signs;
	static std::vector<Link*> links;

	queue.clear();
	signs.clear();
	links.clear();

	applyIdSchemaBinary(x, y, xfs, yfs, signs, links);

	if( ! storeEdge(i, j, signs, links, queue) ) return false;
// 	if( grammar->isUnary() )
// 	    parseUnary(i, j, queue);
	return true;
    }
        
    bool ParserCky::parseK(int i, int k, int j) {
        P2("ParserCky::parseK");
        
        code *oldIP = mach->GetIP();
        core_p TrailP = mach->SetTrailPoint(NULL);
        
                
        std::vector<Edge*>& sik = chart->getEdges(i, k);
        std::vector<Edge*>& skj = chart->getEdges(k, j);
		
            // copy signs in sik and skj
        std::vector<Edge*>::iterator
            v = sik.begin(), vlast = sik.end(),
            w = skj.begin(), wlast = skj.end();
        std::vector<std::pair<Edge*, FSP> > xx, yy;
            
        for(; v != vlast ; v++) {
            P2("ParserCky parseIJ copy v");
            xx.push_back(std::make_pair(*v, FSP(mach, (*v)->getSign())));
        }
        for(; w != wlast ; w++) {
            P2("ParserCky parseIJ copy w");
            yy.push_back(std::make_pair(*w, FSP(mach, (*w)->getSign())));
        }
            
        std::vector<std::pair<Edge*, FSP> >::iterator
            x = xx.begin(), xlast = xx.end(),
            y,              ylast = yy.end();

        for(; x != xlast ; x++) {

            Edge* xe = (*x).first;
            FSP xfs = (*x).second;
            y = yy.begin();
                    
            for(; y != ylast ; y++) {
		Edge* ye = (*y).first;
		FSP yfs = (*y).second;

		if(! parseXY(i, j, xe, xfs, ye, yfs)) return false;
            }
        }
	
        mach->TrailBack(TrailP);
        mach->SetIP(oldIP);
        
        return true;
    }
    void ParserCky::clear() {
        {
            P2("ParserCky::clear chart");
            chart->clear(getSentenceLength());
        }
        {
            P2("ParserCky::clear edge, edgeinfo");
            edgem->clear();
            linkm->clear();
        }
        if( quick_check_path ) {
	    P2("ParserCky::clear qc");
	    qc->clear();
	}
    }

    bool ParserCky::init(machine* m, Grammar* g) {
        ParserBasic::init(m, g);
        
        if ( is_open ) return true;

	fspm = new FspMaker();
        edgem = new EdgeManager();
        linkm = new LinkManager();
//        grammar = new Grammar();
        grammar = g;
        chart = new Chart();
        qc = new QcManager();
        
        if(! fspm->init(m) ) return false;
        if(! chart->init(getLimitWords() + 1) ) return false;
        if(! grammar->init(m) ) return false;
        if(! qc->init(m, grammar) ) return false;
        
	setSentenceLength(0);

        // LiLFeS program for initialization
        module *up_interface;
	// set modules
        if(! (up_interface = module::SearchModule("mayz:parser") )) {
          RUNERR( "module not found: mayz:parser" );
          return false;
	}
        nf_parser_init = 0;
        nf_parser_term = 0;
        parser_init = lilfes::prepare_proc(up_interface, "parser_init", nf_parser_init);
        parser_term = lilfes::prepare_proc(up_interface, "parser_term", nf_parser_term);

        // initialize in lilfes
        if ( parser_init ) {
          IPTrailStack iptrail( m );
          std::vector< FSP > args;
          bool loop = lilfes::call_proc( *m, parser_init, args );
          while(loop)
            loop = mach->NextAnswer();
        }

        is_open=true;
	return true;
    }
    bool ParserCky::term() {
        if(! ParserBasic::term() ) return false;
        if(is_open) {

          // termination in lilfes
          if ( parser_term ) {
            IPTrailStack iptrail( mach );
            std::vector< FSP > args;
            bool loop = lilfes::call_proc( *mach, parser_term, args );
            while(loop)
              loop = mach->NextAnswer();
          }

          // termination of the parser
          if(fspm) delete fspm;
          if(edgem) delete edgem;
          if(linkm) delete linkm;
//          if(grammar) delete grammar;
          if(chart) delete chart;
          if(qc) delete qc;
          delete quick_check_path;
          quick_check_path = NULL;
          do_quick_check = false;
          is_open = false;
        }
        return true;
    }
    
} // end of namespace up

/*
 * $Log: ParserCky.cc,v $
 * Revision 1.4  2005/07/27 09:00:43  yusuke
 *
 * lexical_entry_sign, reduce_sign, fom_XXX, root_sign ϥХåȥå
 * ʤ褦ˤ
 *
 * Revision 1.3  2005/06/21 10:17:12  yusuke
 *
 * copyright 񤭤ޤ
 *
 * Revision 1.2  2005/06/16 07:25:36  yusuke
 *
 * integrate-up 硥
 * version 2.0
 *
 * Revision 1.1.2.16  2005/06/11 04:05:02  yusuke
 *
 * cout -> *lilfes::output_stream
 * cerr -> *lilfes::error_stream
 *
 * Revision 1.1.2.15  2005/06/08 05:59:42  ninomi
 * P2ץեĤʤޤ
 *
 * Revision 1.1.2.14  2005/06/08 05:23:28  yusuke
 *
 * quick check ط
 *
 * Revision 1.1.2.13  2005/06/07 09:31:19  ninomi
 * ParserCky::initΤʤǡParserBasic::initФְ㤨ƤΤ
 * ľޤ
 *
 * Revision 1.1.2.12  2005/06/07 08:59:29  ninomi
 * FS* quick_check_pathtermȤˡä褦ˤޤ
 * grammartermȤ˾äʤ褦ˤޤ
 *
 * Revision 1.1.2.11  2005/06/07 08:49:17  ninomi
 * Parser::init(machine*)init(machine*, Grammar*)Ȥޤ
 *
 * Revision 1.1.2.10  2005/06/07 08:03:12  yusuke
 *
 * grammar_init -> parser_init
 *
 * Revision 1.1.2.9  2005/06/07 07:37:28  ninomi
 * Parserݥ饹ľޤ
 *
 * Revision 1.1.2.8  2005/06/06 16:53:33  yusuke
 *
 * warning ä
 *
 * Revision 1.1.2.7  2005/06/06 16:35:20  yusuke
 *
 * enju ư褦ˤʤޤ
 *
 * Revision 1.1.2.6  2005/06/06 10:39:54  ninomi
 * quick check pathꡢʬ񤭤ޤ
 *
 * Revision 1.1.2.5  2005/06/06 08:53:41  ninomi
 * IPTrailStackطľޤlilfespredicate callʤޤ
 * quick checkidҤˤΤnameǤҤ褦ˤޤ
 *
 * Revision 1.1.2.4  2005/06/05 14:10:48  ninomi
 * ѥ뤬Ȥ褦ˤʤޤ
 *
 * Revision 1.1.2.3  2005/06/05 13:14:21  ninomi
 * Ȥꤢ񴹤ޤǤФäϤ줫
 *
 * Revision 1.1.2.2  2005/06/05 12:19:49  ninomi
 * upmayzβˤĤޤ
 *
 * Revision 1.1.2.1  2005/06/05 10:12:19  ninomi
 * upmayz/srcβ˰ܤϤޤ
 *
 * Revision 1.7  2004/12/09 13:35:58  yusuke
 *
 * factoring  unary rule ŬѤƤޤᡢParserCkyFom 
 * unary rule Ȥȡfom ޤ㤤Ȥ unary rule  fom ׻
 * ޤȤ롣
 *
 * Revision 1.6  2004/11/16 09:14:07  ninomi
 * Artistic LicenseƬˤĤޤ
 *
 * Revision 1.5  2004/07/09 02:11:54  yusuke
 *
 * Parser::getWordLattice 
 * iterative parsing ǡbeam ѤƤʤȤ
 * parseLexiconLookup ʤ褦ˤ
 *
 * Revision 1.4  2004/07/06 13:05:57  ninomi
 * Parser::term()ɲäޤlilfesterminator::PerformAll()¹
 * ɬ¹ԤʤȤʤ
 *
 * Revision 1.3  2004/07/02 03:25:48  ninomi
 * QcFomΣĤΤơ
 * Parser->ParserBasic->ParserCky->ParserCkyQc->ParserFom褦ˤޤQuickCheckenableQuickCheck, disableQuickCheckOn/OffǤޤ
 *
 * Revision 1.2  2004/06/30 13:38:23  ninomi
 * ʸĹsetLimitWordsǡ票åsetLimitEdgesǥѡǤʤ
 * ФĤǤѹǤ褦ˤޤ
 *
 * Revision 1.1  2004/06/30 07:50:29  ninomi
 * up 0.2release versionǤ
 *
 * Revision 1.21  2004/06/01 07:10:22  ninomi
 * ?- (parse(...), printAVM("a") ; printAVM("b"))ޤʤХ
 * ȤΤޤ
 *
 * Revision 1.20  2004/06/01 03:58:02  ninomi
 * ?- (parse(...), printAVM("a") ; printAVM("b").ǤƤޤΤդޤǤޤparsefail"b"Ϥ줺no֤äƤ롣
 *
 * Revision 1.19  2004/05/30 17:11:34  ninomi
 * QuickCheckǤǤEdgeInfoEdgeInfoManagerϿ(registerEdgeInfo())
 * ˺Ƥޤ
 *
 * Revision 1.18  2004/05/26 12:19:12  ninomi
 * FSPΤߤޤäƤ褷ޤ
 *
 * Revision 1.17  2004/05/21 12:44:17  ninomi
 * ncup, qcup, pcup, pqcupƤȤС󤬴ޤ
 *
 * Revision 1.16  2004/05/21 10:50:29  ninomi
 * pcupư褦ˤʤޤ
 *
 * Revision 1.15  2004/05/21 09:19:22  ninomi
 * pcup
 *
 * Revision 1.14  2004/05/21 07:18:03  ninomi
 * pcupγȯǤ
 *
 * Revision 1.13  2004/05/20 23:25:13  ninomi
 * СǤcup, qcupưƤޤpcup, pqcupϥХȤ档
 *
 */
