/*
 * $Id: ParserCky.h,v 1.3 2005/06/21 10:17:12 yusuke Exp $
 *
 *  Copyright (c) 2005, Tsujii Laboratory, The University of Tokyo.
 *  All rights reserved.
 *
 */

#ifndef __ParserCky_h
#define __ParserCky_h

#include "Chart.h"
#include "Fsp.h"
#include "ParserBuiltin.h"
#include <vector>
#include <liblilfes/builtin.h>
#include <liblilfes/machine.h>
#include <liblilfes/ftype.h>
#include <liblilfes/proc.h>
#include <liblilfes/structur.h>
#include <liblilfes/errors.h>
#include <liblilfes/profile.h>
#include <liblilfes/utility.h>
#include <iostream>

namespace up {

    class ParserCky : public ParserBasic {
    private:
        std::vector<Edge*> null_vector;
        bool is_open;
        //bool is_quick_check; // is initialized or not
        bool do_quick_check; // do quick check or not after initialization

        int nf_parser_init;
        int nf_parser_term;
        lilfes::procedure *parser_init;
        lilfes::procedure *parser_term;

    protected:
	FspMaker *fspm;
        EdgeManager *edgem;
        LinkManager *linkm;
        Grammar *grammar;
        Chart *chart;
	QcManager *qc;
        Fs* quick_check_path;
        
        EdgeManager* getEdgeManager() { return edgem; }
        LinkManager* getLinkManager() { return linkm; }

        bool parseLex(const WordLattice& wordlattice);
        bool parsePhrase();
        
	virtual bool ParserCky::storeEdge(int i, int j,
					  std::vector<Fs*>& signs,
					  std::vector<Link*>& einfos,
					  std::vector<Edge*> &queue);

	// lookup lexical entry
        virtual void lookupLexicalEntryCont(int i, int j, lilfes::FSP word, lilfes::FSP lexid, lilfes::FSP sign, lilfes::FSP sign_plus,
					    std::vector<Fs*>& signs, std::vector<Link*>& einfos);
	virtual void lookupLexicalEntry(int i, int j, lilfes::FSP word,
					std::vector<Fs*>& signs, std::vector<Link*>& einfos);

	// id schema unary
	virtual void applyIdSchemaUnaryCont(Edge* e, lilfes::FSP name, lilfes::FSP dtr, lilfes::FSP mother, lilfes::FSP sign_plus,
					    std::vector<Fs*>& signs, std::vector<Link*>& einfos);
	virtual void applyIdSchemaUnary(Edge* e, std::vector<Fs*>& signs, std::vector<Link*>& einfos);
        virtual bool parseUnary(int i, int j, std::vector<Edge*> &queue);

	// id schema binary
	virtual void applyIdSchemaBinaryCont(Edge* x, Edge* y,
					     lilfes::FSP name, lilfes::FSP left, lilfes::FSP right, lilfes::FSP mother, lilfes::FSP sign_plus,
					     std::vector<Fs*>& signs, std::vector<Link*>& einfos);
	virtual void applyIdSchemaBinary(Edge* x, Edge* y,
					 lilfes::FSP xfs, lilfes::FSP yfs,
					 std::vector<Fs*>& signs, std::vector<Link*>& einfos);
	virtual bool parseXY(int i, int j, Edge* x, lilfes::FSP xfs, Edge* y, lilfes::FSP yfs);
        virtual bool parseK(int i, int k, int j);
        //virtual bool parseIJ();
        
    public:
        void setGrammar(Grammar* g) {grammar = g; }
        Grammar* getGrammar() {return grammar; }
        
        void setChart(Chart* c) { chart = c; }
        Chart* getChart() {return chart; }
        
        FspMaker* getFspMaker() { return fspm; }

        std::vector<Edge*>& getEdges(int i, int j) {
            if(0 <= i && i < j && j <= getSentenceLength() && j <= getLimitWords() ) {
                return chart->getEdges(i, j);
            } else {
                return null_vector;
            }
        }
        Edge* getEdge(eserial es) { return edgem->getEdge(es); }
        Link* getLink(lserial is) { return linkm->getLink(is); }
        int getEdgeNumber() { return edgem->getSerialCount(); }

        ParserCky() :is_open(false), do_quick_check(false), fspm(0), edgem(0), linkm(0), grammar(0), chart(0), qc(0), quick_check_path(0) {}
        ~ParserCky() {
            term();
        }

    public:
        bool init(lilfes::machine* m, Grammar*);
        bool term();
        void clear();
        
        //bool isQuickCheck() { return is_quick_check; }
        bool doQuickCheck() { return do_quick_check; }
        void setQuickCheckPath(lilfes::FSP paths) {
            delete quick_check_path;
            quick_check_path = paths.Serialize();
            //is_quick_check = true;
        }
        const Fs* getQuickCheckPath() const {
          return quick_check_path;
        }
        bool enableQuickCheck() {
            if(! quick_check_path) return false;
            if(! qc->setQuickCheckPath(lilfes::FSP(mach, quick_check_path))) return false;
            do_quick_check = true;
            return true;
        }
        void disableQuickCheck() { do_quick_check = false; }

        void setLimitWords(int x) {
          ParserBasic::setLimitWords( x );
          chart->init( x + 1 );
        }

    };
    
} // end of namespace up
#endif // __ParserCky_h

/*
 * $Log: ParserCky.h,v $
 * Revision 1.3  2005/06/21 10:17:12  yusuke
 *
 * copyright 񤭤ޤ
 *
 * Revision 1.2  2005/06/16 07:25:36  yusuke
 *
 * integrate-up 硥
 * version 2.0
 *
 * Revision 1.1.2.13  2005/06/11 04:05:02  yusuke
 *
 * cout -> *lilfes::output_stream
 * cerr -> *lilfes::error_stream
 *
 * Revision 1.1.2.12  2005/06/08 05:23:28  yusuke
 *
 * quick check ط
 *
 * Revision 1.1.2.11  2005/06/08 04:29:09  yusuke
 *
 * get_parser_mode/2 
 *
 * Revision 1.1.2.10  2005/06/07 08:49:17  ninomi
 * Parser::init(machine*)init(machine*, Grammar*)Ȥޤ
 *
 * Revision 1.1.2.9  2005/06/07 08:03:12  yusuke
 *
 * grammar_init -> parser_init
 *
 * Revision 1.1.2.8  2005/06/07 07:37:28  ninomi
 * Parserݥ饹ľޤ
 *
 * Revision 1.1.2.7  2005/06/06 16:35:21  yusuke
 *
 * enju ư褦ˤʤޤ
 *
 * Revision 1.1.2.6  2005/06/06 12:40:50  yusuke
 *
 * ȤꤢѡǤ褦ˤʤä
 *
 * Revision 1.1.2.5  2005/06/06 12:01:02  yusuke
 *
 * Ȥꤢ sample_hpsg  warning Фʤ褦ˤޤ
 *
 * Revision 1.1.2.4  2005/06/06 10:39:54  ninomi
 * quick check pathꡢʬ񤭤ޤ
 *
 * Revision 1.1.2.3  2005/06/06 08:53:41  ninomi
 * IPTrailStackطľޤlilfespredicate callʤޤ
 * quick checkidҤˤΤnameǤҤ褦ˤޤ
 *
 * Revision 1.1.2.2  2005/06/05 14:10:48  ninomi
 * ѥ뤬Ȥ褦ˤʤޤ
 *
 * Revision 1.1.2.1  2005/06/05 10:12:19  ninomi
 * upmayz/srcβ˰ܤϤޤ
 *
 * Revision 1.6  2004/11/16 09:13:54  ninomi
 * Artistic LicenseƬˤĤޤ
 *
 * Revision 1.5  2004/07/12 13:02:18  ninomi
 * sample_hpsgup˲äޤinstallncup up/sample_hpsg.lilǼ¹
 *
 * Revision 1.4  2004/07/09 02:11:54  yusuke
 *
 * Parser::getWordLattice 
 * iterative parsing ǡbeam ѤƤʤȤ
 * parseLexiconLookup ʤ褦ˤ
 *
 * Revision 1.3  2004/07/06 13:05:50  ninomi
 * Parser::term()ɲäޤlilfesterminator::PerformAll()¹
 * ɬ¹ԤʤȤʤ
 *
 * Revision 1.2  2004/06/30 13:38:15  ninomi
 * ʸĹsetLimitWordsǡ票åsetLimitEdgesǥѡǤʤ
 * ФĤǤѹǤ褦ˤޤ
 *
 * Revision 1.1  2004/06/30 07:50:09  ninomi
 * up 0.2release versionǤ
 *
 */

