/*
 * $Id: ParserCkyFom.cc,v 1.7 2005/07/27 09:00:43 yusuke Exp $
 *
 *  Copyright (c) 2005, Tsujii Laboratory, The University of Tokyo.
 *  All rights reserved.
 *
 */

#include "ParserCkyFom.h"
#include <cfloat>

namespace up {
    using namespace lilfes;

    FomManager* ParserCkyFom::edge_sort_criterion::fomm;
    FomManager* ParserCkyFom::link_sort_criterion::fomm;

    void ParserCkyFom::globalThresholdingForward(int n) {
        for(int start = 0 ; start <= n-1 ; ++start) {
            double left = f[start];
            for(int j = start+1 ; j <= n ; ++j) {
                std::vector<Edge*>& sij = chart->getEdges(start, j);
                if(sij.size() > 0) {
                    double score = left + (fomm->getEdgeFom(sij.front()->getSerialNo()));
                    if( score > f[j] ) f[j] = score;
                }
            }
        }
    }
    void ParserCkyFom::globalThresholdingBackward(int n) {
        for(int start = n - 1 ; start >= 0 ; start--) {
            for(int j = start+1 ; j <= n ; j++) {
                std::vector<Edge*>& sij = chart->getEdges(start, j);
                if(sij.size() > 0) {
                    double right = b[j];
                    double score = right + (fomm->getEdgeFom(sij.front()->getSerialNo()));
                    if( score > b[start] ) b[start] = score;
                }
            }
        }
        
    }
    void ParserCkyFom::globalThresholding(int len, int n) {
        P2("ParserCkyFom::globalThresholding");

        f.clear(); f.resize(n+1, -1000000.0); f[0] = 0.0;
        b.clear(); b.resize(n+1, -1000000.0); b[n] = 0.0;
        globalThresholdingForward(n);
        globalThresholdingBackward(n);
        double bestprob = f[n];
        
        for(int i = 0 ; i < n ; i++)
            for(int j = i+1; j <= i+len && j <= n ; j++) {
                std::vector<Edge*>& sij = chart->getEdges(i, j);
                std::vector<Edge*>::iterator it = sij.begin(), last = sij.end();
                for(; it != last ; ++it) {
                    double left = f[i];
                    double right = b[j];
                    double total = left + (fomm->getEdgeFom((*it)->getSerialNo())) + right;
                    if( total < bestprob - global_beam_threshold_width)
                        break;
                }
                for(; it != last ; ++it) {
                    eserial edgeid = (*it)->getSerialNo();
                    if(edge_state[edgeid] > current_state) break;
                    edge_state[edgeid] = current_state + 1;
                }
            }
    }

    void ParserCkyFom::globalThresholdingLight(int len, int n) {
        P2("ParserCkyFom::globalThresholdingLight");

        double best = -1000000;
        for(int i = 0 ; i <= n - len; ++i) {
            int j = i + len;
            std::vector<Edge*>& sij = chart->getEdges(i, j);
            if(sij.size() > 0) {
                double score = fomm->getEdgeFom(sij.front()->getSerialNo());
                if( score > best ) best = score;
            }
        }
        for(int i = 0 ; i <= n - len; ++i) {
            int j = i + len;
            std::vector<Edge*>& sij = chart->getEdges(i, j);
            std::vector<Edge*>::iterator it = sij.begin(), last = sij.end();
            for(; it != last ; ++it) {
                double score = fomm->getEdgeFom((*it)->getSerialNo());
                if( score < best - global_beam_threshold_width)
                    break;
            }
            for(; it != last ; ++it)
                edge_state[(*it)->getSerialNo()] = current_state + 1;
            
        }
    }
    int ParserCkyFom::thresholdOutEdges(std::vector<Edge*>& sij,
					const int num,
					const double width) {
        P2("ParserCkyFom::thresholdOutEdges");
        
        int size = sij.size();
        if (size == 0) return 0;

        edge_state.resize(edgem->getSerialCount(), current_state);
        
        int num_pruned = 0;
        sort(sij.begin(), sij.end(), edge_sort_criterion());

        const double max_fom = fomm->getEdgeFom(sij.front()->getSerialNo());
        if ( num > 0 ) {
            for (int k = 0; k < size - num; k++) {
                int target = size - num_pruned - 1;
                edge_state[sij[target]->getSerialNo()] = current_state + 1;
                num_pruned++;
            }
        }

        if ( width > 0.0 ) {
            while (1) {
                assert(sij.size() > 0);
                int target = size - num_pruned - 1;
                if (fomm->getEdgeFom(sij[target]->getSerialNo()) < max_fom - width) {
                    edge_state[sij[target]->getSerialNo()] = current_state + 1;
                    num_pruned++;
                } else {
                    break;
                }
            }
        }
        return num_pruned;
    }
    int ParserCkyFom::thresholdOutPreLexes(std::vector<PreLex*>& sij,
                                          const int num,
                                          const double width) {
        P2("ParserCkyFom::thresholdOutPreLexes");
        
        int size = sij.size();
        if (size == 0) return 0;

        plex_state.resize(plexm->getSerialCount(), current_state);
        
        int num_pruned = 0;
        sort(sij.begin(), sij.end(), plex_sort_criterion());

        const double max_fom = sij.front()->getFom();
        if ( num > 0 ) {
            for (int k = 0; k < size - num; k++) {
                int target = size - num_pruned - 1;
                plex_state[sij[target]->getSerialNo()] = current_state + 1;
                num_pruned++;
            }
        }

        if ( width > 0.0 ) {
            while (1) {
                assert(sij.size() > 0);
                int target = size - num_pruned - 1;
                if (sij[target]->getFom() < max_fom - width) {
                    plex_state[sij[target]->getSerialNo()] = current_state + 1;
                    num_pruned++;
                } else {
                    break;
                }
            }
        }
        return num_pruned;
    }

    bool ParserCkyFom::storeEdge(int i, int j,
				 std::vector<Fs*>& signs,
				 std::vector<Link*>& einfos,
				 std::vector<Edge*> &queue) {
        P2("ParserCkyFom::StoreEdge");
        ASSERT(0 <= i);
        ASSERT(i < j);
        ASSERT(j < (limit_words + 1));

        std::vector<Link*>::iterator it = einfos.begin(), last = einfos.end();
	std::vector<Fs*>::iterator it_sign = signs.begin();
    
        for(; it != last ; ++it, ++it_sign) {

            Fs* x = *it_sign;
        
            std::vector<Edge*> &xxx = chart->getEdges(i, j);
            std::vector<Edge*>::iterator cell_it = xxx.begin(), cell_last = xxx.end();

            bool factored = false;
                // if factored, store edge_info
            for(; cell_it != cell_last ; ++cell_it) {
                Edge* e = (*cell_it);
                Fs* y = e->getSign();
                if ( (*x) == (*y) ) {
                    e->addLink(*it);
		    eserial esn = e->getSerialNo();
                    (*it)->setMyEdgeSerialNo(esn);
                    delete x;
                    factored = true;

		    // fom
		    lserial isn = (*it)->getSerialNo();
                    double new_fom = std::max(fomm->getEdgeFom(esn),
                                              fomm->getLinkFom(isn));
                    if ( new_fom > max_fom ) max_fom = new_fom;
		    fomm->setEdgeFom(esn, new_fom);
                    break;
                }
            }

                // not factored
            if (! factored ) {

                if(((int) getEdgeManager()->getSerialCount()) >= getLimitEdges() ) {
//                    std::cout << "over edge limit limit: " << getLimitEdges() << " serial count: " << getEdgeManager()->getSerialCount() << std::endl;
                    delete x; return false;
                }

                Edge *e = edgem->makeEdge(std::pair<int, int>(i, j), x);
		eserial esn = e->getSerialNo();
                
                e->addLink(*it);
                (*it)->setMyEdgeSerialNo(esn);
                
                chart->getEdges(i, j).push_back(e);
                queue.push_back(e);
		// fom
		lserial isn = (*it)->getSerialNo();
                double new_fom = fomm->getLinkFom(isn);
                if ( new_fom > max_fom ) max_fom = new_fom;
		fomm->setEdgeFom(esn, new_fom);
            }
        }
        return true;
    }

  void ParserCkyFom::lookupLexicalEntryCont(int i, int j, FSP word, FSP lexid, FSP sign, FSP sign_plus,
					      std::vector<Fs*>& signs, std::vector<Link*>& einfos) {
        P2("ParserCkyFom::lookupLexicalEntryCont");
	IPTrailStack iptrail(mach);
	FSP fom1(mach); FSP fom2(mach); //FSP reduction(mach);
        
        bool loop = fomm->fomLexicalEntry(word, lexid, fom1) && fomm->fomTerminal(lexid, sign, sign_plus, fom2);
        for(; loop ; loop = mach->NextAnswer() ) {
            ParserCky::lookupLexicalEntryCont(i, j, word, lexid, sign, sign_plus, signs, einfos);
            fomm->setLinkFom(einfos.back()->getSerialNo(), (double) fom1.ReadFloat() + (double) fom2.ReadFloat() );
//             fomm->setLinkReduction(einfos.back()->getSerialNo(), reduction);
        }
    }
    
    void ParserCkyFom::applyIdSchemaUnaryCont(Edge* e, FSP name, FSP dtr, FSP mother, FSP sign_plus,
					      std::vector<Fs*>& signs, std::vector<Link*>& einfos) {
        P2("ParserCkyFom::applyIdSchemaUnaryCont");
        //IPTrailStack iptrail(mach);
	FSP dfom(mach); //FSP rmother(mach); FSP reduction(mach);
        bool loop = fomm->fomUnary(name, dtr, mother, sign_plus, dfom);
        if ( ! loop ) return;
        //for( ; loop ; loop = mach->NextAnswer() ) {
        ParserCky::applyIdSchemaUnaryCont(e, name, dtr, mother, sign_plus, signs, einfos);
        //std::cerr << name.DisplayAVM();
        //std::cerr << e->getSerialNo() << " " << dtr.DisplayAVM();
        //std::cerr << dfom.ReadFloat() << " + " << fomm->getEdgeFom(e->getSerialNo()) << std::endl;
//                    double d = dfom.ReadFloat();
//                    if(d > 0.0) {
//                        dfom_total += d;
//                        dfom_count++;
//                    }
        fomm->setLinkFom(einfos.back()->getSerialNo(),
                         (double) (dfom.ReadFloat() +
                                   fomm->getEdgeFom(e->getSerialNo())));
//             fomm->setLinkReduction(einfos.back()->getSerialNo(), reduction);
        //}
    }

    void ParserCkyFom::applyIdSchemaBinaryCont(Edge* x, Edge* y,
					       FSP name, FSP left, FSP right, FSP mother, FSP sign_plus,
					       std::vector<Fs*>& signs, std::vector<Link*>& einfos) {
        P2("ParserCkyFom::applyIdSchemaBinaryCont");
	//IPTrailStack iptrail(mach);
	FSP dfom(mach); //FSP rmother(mach); FSP reduction(mach);
        
        bool loop = fomm->fomBinary(name, left, right, mother, sign_plus, dfom);
        if ( ! loop ) return;
        //for(; loop ; loop = mach->NextAnswer() ) {
//             P2("ParserCkyFom::applyIdSchemaBinaryCont-fomEquivalentReduction");
        ParserCky::applyIdSchemaBinaryCont(x, y, name, left, right, mother, sign_plus, signs, einfos);
//                    double d = dfom.ReadFloat();
//                    if(d > 0.0) {
//                        dfom_total += d;
//                        dfom_count++;
//                    }
        fomm->setLinkFom(einfos.back()->getSerialNo(),
                         (double) (dfom.ReadFloat() +
                                   fomm->getEdgeFom(x->getSerialNo()) +
                                   fomm->getEdgeFom(y->getSerialNo())));
//             fomm->setLinkReduction(einfos.back()->getSerialNo(), reduction);
        //}
    }

    bool ParserCkyFom::parseXY(int i, int j, Edge* x, FSP xfs, Edge*y, FSP yfs) {
	P2("ParserCkyFom::parseXY");

        if((edge_state[x->getSerialNo()] != current_state) &&
           (edge_state[y->getSerialNo()] != current_state)) return true;
        
	static std::vector<Edge*> queue;
	static std::vector<Fs*> signs;
	static std::vector<Link*> einfos;

	queue.clear();
	signs.clear();
	einfos.clear();

	applyIdSchemaBinary(x, y, xfs, yfs, signs, einfos);

	if( ! storeEdge(i, j, signs, einfos, queue) ) return false;
	return true;
    }
    
    bool ParserCkyFom::parseK(int i, int k, int j) {
        P2("ParserCkyFom::parseK");
        
        IPTrailStack iptrail(mach);
                
        std::vector<Edge*>& sik = chart->getEdges(i, k);
        std::vector<Edge*>& skj = chart->getEdges(k, j);
		
            // copy signs in sik and skj
        std::vector<Edge*>::iterator
            v = sik.begin(), vlast = sik.end(),
            w = skj.begin(), wlast = skj.end();
        std::vector<std::pair<Edge*, FSP> > xx, yy;
            
        for(; v != vlast ; v++) {
            P2("ParserCky parseIJ copy v");
            if(edge_state[(*v)->getSerialNo()] <= current_state) {
                xx.push_back(std::make_pair(*v, FSP(mach, (*v)->getSign())));
            } else { break; }
        }
        for(; w != wlast ; w++) {
            P2("ParserCky parseIJ copy w");
            if(edge_state[(*w)->getSerialNo()] <= current_state) {
                yy.push_back(std::make_pair(*w, FSP(mach, (*w)->getSign())));
            } else { break; }
        }
            
        std::vector<std::pair<Edge*, FSP> >::iterator
            x = xx.begin(), xlast = xx.end(),
            y,              ylast = yy.end();

        for(; x != xlast ; x++) {

            Edge* xe = (*x).first;
            FSP xfs = (*x).second;
            y = yy.begin();
                    
            for(; y != ylast ; y++) {
		Edge* ye = (*y).first;
		FSP yfs = (*y).second;

		if(! parseXY(i, j, xe, xfs, ye, yfs)) return false;
            }
        }
        
        return true;
    }
    
    bool ParserCkyFom::parseLex(const WordLattice& wordlattice) {
        P2("ParserCkyFom::parseLex");
        if(!isPreLex()) return ParserCky::parseLex(wordlattice);

        WordLattice::const_iterator it = wordlattice.begin(), last = wordlattice.end();
        for(; it != last ; it++) {
	    ExtentWord *exw = dynamic_cast<ExtentWord*>(*it);
            if(!exw) continue;
            
            IPTrailStack iptrail(mach);
            
            FSP word(mach, exw->getWord());
            FSP lexid(mach); FSP fom(mach);
            bool loop = grammar->lexicalEntry(word, lexid);
            for(; loop ; loop = mach->NextAnswer() ) {
                //IPTrailStack iptrail2(mach);
                bool loop2 = fomm->fomLexicalEntry(word, lexid, fom);
                if ( ! loop2 ) continue;
                //for(; loop2 ; loop2 = mach->NextAnswer() ) {
                PreLex *p = plexm->makePreLex(exw, lexid.Serialize(), exw->getWord(), fom.ReadFloat());
                chart->addPreLex(exw->getLeftPosition(), exw->getRightPosition(), p);
                //}
            }
        }
        return true;
    }

    bool ParserCkyFom::delayedLookupLexicalEntry(int i, int j, PreLex* plex) {
        P2("ParserCkyFom::delayedLookupLexicalEntry");
        static std::vector<Edge*> queue;
        static std::vector<Fs*> signs;
        static std::vector<Link*> links;

        queue.clear();
        signs.clear();
        links.clear();

        IPTrailStack iptrail(mach);
            
        FSP lexid(mach, plex->getLexID());
        FSP word(mach, plex->getWord());
        FSP sign(mach); FSP reduced_sign(mach); FSP sign_plus(mach);

        bool loop2 = grammar->lexicalEntrySign(lexid, sign);
        //for(; loop2 ; loop2 = mach->NextAnswer() ) {
        if ( loop2 ) {
            //IPTrailStack iptrail3(mach);
            bool loop3 = grammar->reduceSign(sign, reduced_sign, sign_plus);
            //for(; loop3 ; loop3 = mach->NextAnswer() ) {
            if ( loop3 ) {
                //IPTrailStack iptrail4(mach);
                FSP fom2(mach);
                bool loop4 = fomm->fomTerminal(lexid, sign, sign_plus, fom2);
                //for(; loop4 ; loop4 = mach->NextAnswer() ) {
                if ( loop4 ) {
                    ParserCky::lookupLexicalEntryCont(i, j, word, lexid, sign, sign_plus, signs, links);
                    fomm->setLinkFom(links.back()->getSerialNo(), plex->getFom() + (double) fom2.ReadFloat() );
                }
            }
        }
        if(! storeEdge(i, j, signs, links, queue) ) return false;
        if( grammar->isUnary() )
            parseUnary(i, j, queue);
        return true;
    }
    
    bool ParserCkyFom::parsePhrase() {
        P2("ParserCkyFom::parsePhrase");
        
	int n = getSentenceLength();

        if(isPreLex()) {
                // cut terminal edges
            for(int i = 0 ; i < n ; i++)
                for(int j = i + 1; j <= n ; j++) {
                    std::vector<PreLex*>& sij = chart->getPreLexes(i, j);
                    if(sij.size() > 0) {
                        thresholdOutPreLexes(sij, beam_threshold_num, beam_threshold_width);
                    }
                }

                // restore terminal
            std::vector<Edge*> queue;
            for(int i = 0 ; i < n ; i++)
                for(int j = i + 1; j <= n ; j++) {
                    std::vector<PreLex*>& sij = chart->getPreLexes(i, j);
                    if(sij.size() > 0) {
                        queue.clear();

                        std::vector<PreLex*>::iterator it = sij.begin(), last = sij.end();
                        for(; it != last; ++it) {
                            int state = plex_state[(*it)->getSerialNo()];
                            if( state > current_state) break;
                            if( state == current_state)
                                delayedLookupLexicalEntry(i, j, (*it));
                        }
                    }
                }
        }
        
        for( int len = 1; len <= n ; len++) {
            int maxi = n - len;
            for(int i = 0 ; i <= maxi; i++) {
                int j = i + len;

		if(! chart->isAlive(i,j) ) continue;

                max_fom = -DBL_MAX;

		for( int k = i + 1; k < j ; k++ )
                  if(!parseK(i,k,j)) return false;
                    
                if( grammar->isUnary() ) {
                  std::vector<Edge*> queue = chart->getEdges(i, j);
                  parseUnary(i, j, queue);
                }

		std::vector<Edge*>& sij = chart->getEdges(i, j);

		// beam_thresholding
		thresholdOutEdges(sij, beam_threshold_num, beam_threshold_width);

		// sort edge_infos
		std::vector<Edge*>::iterator it = sij.begin(), last = sij.end();
		for (; it != last ; it++) {
		    std::vector<Link*> & ei = (*it)->getLinks();
		    sort(ei.begin(), ei.end(), link_sort_criterion());
		}
	    }

            if(isGlobalThresholding())
                globalThresholding(len, n);
            
            if(isGlobalThresholdingLight())
                globalThresholdingLight(len, n);
        }
        
            // fom root
	std::vector<Edge*>& root = chart->getEdges(0, n);
	std::vector<Edge*>::iterator it = root.begin(), last = root.end();
	for ( ; it != last ; it++) {
            IPTrailStack iptrail(mach);
	    FSP sign(mach, (*it)->getSign()); FSP fom(mach);
	    bool loop = fomm->fomRoot(sign, fom);
	    //for(; loop ; loop = mach->NextAnswer() ) {
            if ( loop ) {
		eserial esn = (*it)->getSerialNo();
		fomm->setEdgeFom(esn,
				 fomm->getEdgeFom(esn) +
				 (double) fom.ReadFloat());
	    }
	}
	sort(root.begin(), root.end(), edge_sort_criterion());
	//  if (!root.empty())
	//    cerr << "best_fom = " << root.front()->GetFom() << " ";
	//  cerr << "finished Parse()." << endl;
//        std::cout << "dfom_total: " << dfom_total << " dfom_count: " << dfom_count << std::endl;
	return true;
    }

    void ParserCkyFom::clear() {
	ParserCky::clear();
	{
	    P2("ParserCkyFom::clear");
	    fomm->clear();
            plexm->clear();
            edge_state.clear();
            plex_state.clear();
            current_state = 0;
	}
    }
    bool ParserCkyFom::init(machine* m, Grammar* g) {
        if(! ParserCky::init(m, g)) return false;
        plexm = new PreLexManager();
	fomm = new FomManager();
	edge_sort_criterion::fomm = fomm;
	link_sort_criterion::fomm = fomm;
	if(!fomm->init(m)) return false;
        is_open = true;
	return true;
    }
    bool ParserCkyFom::term() {
        if(! ParserCky::term()) return false;
        if(is_open) {
            if(plexm) {
                delete plexm;
            }
            if(fomm) {
                delete fomm;
            }
            is_open = false;
        }
        return true;
    }

} // end of namespace up

/*
 * $Log: ParserCkyFom.cc,v $
 * Revision 1.7  2005/07/27 09:00:43  yusuke
 *
 * lexical_entry_sign, reduce_sign, fom_XXX, root_sign ϥХåȥå
 * ʤ褦ˤ
 *
 * Revision 1.6  2005/06/29 08:23:28  ninomi
 * prelexεǽon/offǤ褦ˤޤ
 *
 * Revision 1.5  2005/06/26 09:05:59  ninomi
 * PreLex򤤤줿С
 *
 * Revision 1.4  2005/06/25 06:11:41  ninomi
 * pre-lexicon-lookup implementation
 *
 * Revision 1.3  2005/06/21 10:17:12  yusuke
 *
 * copyright 񤭤ޤ
 *
 * Revision 1.2  2005/06/16 07:25:36  yusuke
 *
 * integrate-up 硥
 * version 2.0
 *
 * Revision 1.1.2.7  2005/06/09 00:16:10  yusuke
 *
 * extract_???_event  fom_??? ˤ碌ƥ󥿥եѹ
 *
 * Revision 1.1.2.6  2005/06/08 05:59:42  ninomi
 * P2ץեĤʤޤ
 *
 * Revision 1.1.2.5  2005/06/07 08:49:17  ninomi
 * Parser::init(machine*)init(machine*, Grammar*)Ȥޤ
 *
 * Revision 1.1.2.4  2005/06/06 16:35:21  yusuke
 *
 * enju ư褦ˤʤޤ
 *
 * Revision 1.1.2.3  2005/06/06 08:53:41  ninomi
 * IPTrailStackطľޤlilfespredicate callʤޤ
 * quick checkidҤˤΤnameǤҤ褦ˤޤ
 *
 * Revision 1.1.2.2  2005/06/05 14:10:48  ninomi
 * ѥ뤬Ȥ褦ˤʤޤ
 *
 * Revision 1.1.2.1  2005/06/05 12:19:49  ninomi
 * upmayzβˤĤޤ
 *
 * Revision 1.23  2005/06/03 04:58:38  ninomi
 * Ȥꤢglobal thresholdingǤreleaseǤ
 *
 * Revision 1.22  2005/06/02 08:41:27  ninomi
 * localthresholding on/offäޤɤޤ
 *
 * Revision 1.21  2005/06/01 12:35:05  ninomi
 * commentäޤ
 *
 * Revision 1.20  2005/06/01 08:53:26  ninomi
 * ɤbug fix
 *
 * Revision 1.19  2005/06/01 07:38:02  ninomi
 * ̤iterative+global thresholding left cornerȤȤȤޤΤľޤ
 *
 * Revision 1.18  2005/06/01 02:34:57  ninomi
 * global-thresholding-left-cornerޤ
 *
 * Revision 1.17  2005/05/31 14:06:20  ninomi
 * äȤŬޤ
 *
 * Revision 1.16  2005/05/31 13:40:20  ninomi
 * global thresholding light ޤ
 *
 * Revision 1.15  2005/05/31 12:21:07  ninomi
 * global thresholding algorithmɲäޤ
 *
 * Revision 1.14  2005/03/02 08:08:45  yusuke
 *
 * FOM γˤơ롼ŬѤ򤹤뤫ɤꤹ ParserCkyPred ä
 *
 * Revision 1.13.2.1  2005/02/26 16:50:51  yusuke
 *
 * ParserCkyPred ä
 *
 * Revision 1.13  2005/02/26 12:18:16  yusuke
 *
 * edge_info γΨȡfom_equivalent_reduction 裳¸ơ
 * edge_info_fom 褦ˤ
 *
 * Revision 1.12  2004/12/09 13:35:58  yusuke
 *
 * factoring  unary rule ŬѤƤޤᡢParserCkyFom 
 * unary rule Ȥȡfom ޤ㤤Ȥ unary rule  fom ׻
 * ޤȤ롣
 *
 * Revision 1.11  2004/11/16 09:14:07  ninomi
 * Artistic LicenseƬˤĤޤ
 *
 * Revision 1.10  2004/09/17 14:59:33  yusuke
 *
 * threshold  0 ʲΤȤ thresholding 򤷤ʤ褦ˤ
 *
 * Revision 1.9  2004/07/07 13:10:03  ninomi
 * warningäΤǾäޤ
 *
 * Revision 1.8  2004/07/06 13:21:21  ninomi
 * semanticPrincipleBinaryfomBiaryfomEquivalentReductionˤ֤P2
 * ǻꤷƤޤ
 *
 * Revision 1.7  2004/07/06 13:05:57  ninomi
 * Parser::term()ɲäޤlilfesterminator::PerformAll()¹
 * ɬ¹ԤʤȤʤ
 *
 * Revision 1.6  2004/07/02 08:07:45  ninomi
 * ParserCkyFomiterative parseεǽParserCkyIter˰ܤޤ
 *
 * Revision 1.5  2004/07/02 03:25:49  ninomi
 * QcFomΣĤΤơ
 * Parser->ParserBasic->ParserCky->ParserCkyQc->ParserFom褦ˤޤQuickCheckenableQuickCheck, disableQuickCheckOn/OffǤޤ
 *
 * Revision 1.4  2004/07/01 07:16:24  yusuke
 *
 * root condition 򤷤٤Ȥ pushIpTrail Ĥ褦ˤ
 *
 * Revision 1.3  2004/06/30 16:19:15  yusuke
 *
 * iterative parse 
 *
 * Revision 1.2  2004/06/30 13:38:23  ninomi
 * ʸĹsetLimitWordsǡ票åsetLimitEdgesǥѡǤʤ
 * ФĤǤѹǤ褦ˤޤ
 *
 * Revision 1.1  2004/06/30 07:50:29  ninomi
 * up 0.2release versionǤ
 *
 */
