/*
 * $Id: ParserCkyFom.h,v 1.5 2005/06/29 08:23:28 ninomi Exp $
 *
 *  Copyright (c) 2005, Tsujii Laboratory, The University of Tokyo.
 *  All rights reserved.
 *
 */

#ifndef __ParserCkyFom_h
#define __ParserCkyFom_h

#include "ParserCky.h"
#include "Fom.h"
#include <vector>
#include <liblilfes/machine.h>
#include <liblilfes/ftype.h>
#include <liblilfes/proc.h>
#include <liblilfes/structur.h>

namespace up {
    class ParserCkyFom : public ParserCky {
    private:
        bool is_open;
        int is_global_thresholding; // 0 = disabled, 1 = global_thresholding_light, 2 = global_thresholding
        bool is_plex;

        int current_state; // always 0 in ParserCkyFom
                           // the current iteration number in ParserCkyIter
        std::vector<int> edge_state; // 0 = alive, 1 = dead in ParserCkyFom
                                     // the iteration number when the edge is generated
        std::vector<int> plex_state;
    protected:
        std::vector<double> f;
        std::vector<double> b;
        int beam_threshold_num;
        double beam_threshold_width;
        double global_beam_threshold_width;
        double max_fom;

        void incCurrentState() { current_state++; }

//        double dfom_total;
//        int dfom_count;

        PreLexManager *plexm;
        FomManager *fomm;
        struct edge_sort_criterion {
	    static FomManager *fomm;
            bool operator()(const Edge* e1, const Edge* e2) const {
                return fomm->getEdgeFom(e1->getSerialNo()) > fomm->getEdgeFom(e2->getSerialNo());
            }
        };
        
        struct link_sort_criterion {
	    static FomManager *fomm;
            bool operator()(const Link* e1, const Link* e2) const {
                return fomm->getLinkFom(e1->getSerialNo()) > fomm->getLinkFom(e2->getSerialNo());
            }
        };
        struct plex_sort_criterion {
            bool operator()(const PreLex* e1, const PreLex* e2) const {
                return e1->getFom() > e2->getFom();
            }
        };
        void globalThresholdingForward(int n);
        void globalThresholdingBackward(int n);
        
        virtual void globalThresholding(int len, int n);
        virtual void globalThresholdingLight(int len, int n);
        virtual int thresholdOutEdges(std::vector<Edge*>& sij, const int num, const double width);
        virtual int thresholdOutPreLexes(std::vector<PreLex*>& sij, const int num, const double width);
	bool storeEdge(int i, int j,
		       std::vector<Fs*>& signs,
		       std::vector<Link*>& einfos,
		       std::vector<Edge*> &queue);
        void lookupLexicalEntryCont(int i, int j, lilfes::FSP word, lilfes::FSP lexid, lilfes::FSP sign, lilfes::FSP sign_plus,
				    std::vector<Fs*>& signs, std::vector<Link*>& einfos);
	void applyIdSchemaUnaryCont(Edge* e, lilfes::FSP name, lilfes::FSP dtr, lilfes::FSP mother, lilfes::FSP sign_plus,
				    std::vector<Fs*>& signs, std::vector<Link*>& einfos);
	void applyIdSchemaBinaryCont(Edge* x, Edge* y,
				     lilfes::FSP name, lilfes::FSP left, lilfes::FSP right, lilfes::FSP mother, lilfes::FSP sign_plus,
				     std::vector<Fs*>& signs, std::vector<Link*>& einfos);
        bool parseXY(int i, int j, Edge* x, lilfes::FSP xfs, Edge*y, lilfes::FSP yfs);
        bool parseK( int i, int k, int j );
        bool parseLex(const WordLattice& wordlattice);
        bool delayedLookupLexicalEntry(int i, int j, PreLex* plex);
        bool parsePhrase();
        //virtual bool __parseIJ( int beam_num, double beam_width );
        bool __init(lilfes::machine* m, FomManager* fm);
        bool __term();
    public:
        ParserCkyFom()
                :
                is_open(false),
                is_global_thresholding(0),
                is_plex(false),
                current_state(0),
                    // default settings for beam thresholding
                    //static int beam_threshold_num = 999999;
                beam_threshold_num(12),
                    //static double beam_threshold_width = 999999;
                beam_threshold_width(6.0),
                global_beam_threshold_width(6.0),
//                dfom_total(0.0),
//                dfom_count(0),
                plexm(0),
                fomm(0) {}
        ~ParserCkyFom() { term(); }
	void clear();
        bool init(lilfes::machine* m, Grammar* g);
        bool term();
        int getBeamThresholdNum() { return beam_threshold_num; }
        void setBeamThresholdNum(int b) { beam_threshold_num = b; }
        double getBeamThresholdWidth() { return beam_threshold_width; }
        void setBeamThresholdWidth(double b) { beam_threshold_width = b; }
        double getGlobalBeamThresholdWidth() { return global_beam_threshold_width; }
        void setGlobalBeamThresholdWidth(double b) { global_beam_threshold_width = b; }

	double getEdgeFom(eserial esn) { return fomm->getEdgeFom(esn); }
	double getLinkFom(lserial lsn) { return fomm->getLinkFom(lsn); }
	double getFom(Edge* e) { return fomm->getEdgeFom(e->getSerialNo()); }
	double getFom(Link* i) { return fomm->getLinkFom(i->getSerialNo()); }
//         const std::vector< lilfes::cell >& getReduction(Link* i) { return fomm->getLinkReduction(i->getSerialNo()); }
        FomManager* getFomManager() { return fomm; }

        bool isGlobalThresholding() { return (is_global_thresholding == 2); }
        void enableGlobalThresholding() { is_global_thresholding = 2; }
        void disableGlobalThresholding() { is_global_thresholding = 0; }
        
        bool isGlobalThresholdingLight() { return (is_global_thresholding == 1); }
        void enableGlobalThresholdingLight() { is_global_thresholding = 1; }
        void disableGlobalThresholdingLight() { is_global_thresholding = 0; }
        
        bool isPreLex() { return is_plex; }
        void enablePreLex() { is_plex = true; }
        void disablePreLex() { is_plex = false; }
        
    };
}
#endif // __ParserCkyFom_h

/*
 * $Log: ParserCkyFom.h,v $
 * Revision 1.5  2005/06/29 08:23:28  ninomi
 * prelexεǽon/offǤ褦ˤޤ
 *
 * Revision 1.4  2005/06/25 06:11:41  ninomi
 * pre-lexicon-lookup implementation
 *
 * Revision 1.3  2005/06/21 10:17:12  yusuke
 *
 * copyright 񤭤ޤ
 *
 * Revision 1.2  2005/06/16 07:25:36  yusuke
 *
 * integrate-up 硥
 * version 2.0
 *
 * Revision 1.1.2.5  2005/06/07 08:49:17  ninomi
 * Parser::init(machine*)init(machine*, Grammar*)Ȥޤ
 *
 * Revision 1.1.2.4  2005/06/06 16:35:21  yusuke
 *
 * enju ư褦ˤʤޤ
 *
 * Revision 1.1.2.3  2005/06/06 12:01:02  yusuke
 *
 * Ȥꤢ sample_hpsg  warning Фʤ褦ˤޤ
 *
 * Revision 1.1.2.2  2005/06/05 14:10:48  ninomi
 * ѥ뤬Ȥ褦ˤʤޤ
 *
 * Revision 1.1.2.1  2005/06/05 12:19:49  ninomi
 * upmayzβˤĤޤ
 *
 * Revision 1.16  2005/06/03 04:58:32  ninomi
 * Ȥꤢglobal thresholdingǤreleaseǤ
 *
 * Revision 1.15  2005/06/02 08:41:21  ninomi
 * localthresholding on/offäޤɤޤ
 *
 * Revision 1.14  2005/06/01 07:37:50  ninomi
 * ̤iterative+global thresholding left cornerȤȤȤޤΤľޤ
 *
 * Revision 1.13  2005/06/01 02:34:49  ninomi
 * global-thresholding-left-cornerޤ
 *
 * Revision 1.12  2005/05/31 13:40:11  ninomi
 * global thresholding light ޤ
 *
 * Revision 1.11  2005/05/31 12:20:57  ninomi
 * global thresholding algorithmɲäޤ
 *
 * Revision 1.10  2005/05/29 10:50:58  ninomi
 * iterative parsingǳiterationclearΤǤϤʤiterationη̤Ѥversion (ParserCkyReuseIter)
 *
 * Revision 1.9  2005/03/02 08:08:44  yusuke
 *
 * FOM γˤơ롼ŬѤ򤹤뤫ɤꤹ ParserCkyPred ä
 *
 * Revision 1.8.2.1  2005/02/26 16:50:51  yusuke
 *
 * ParserCkyPred ä
 *
 * Revision 1.8  2005/02/26 12:18:12  yusuke
 *
 * edge_info γΨȡfom_equivalent_reduction 裳¸ơ
 * edge_info_fom 褦ˤ
 *
 * Revision 1.7  2004/11/16 09:13:54  ninomi
 * Artistic LicenseƬˤĤޤ
 *
 * Revision 1.6  2004/07/06 13:05:50  ninomi
 * Parser::term()ɲäޤlilfesterminator::PerformAll()¹
 * ɬ¹ԤʤȤʤ
 *
 * Revision 1.5  2004/07/02 08:07:37  ninomi
 * ParserCkyFomiterative parseεǽParserCkyIter˰ܤޤ
 *
 * Revision 1.4  2004/07/02 03:25:33  ninomi
 * QcFomΣĤΤơ
 * Parser->ParserBasic->ParserCky->ParserCkyQc->ParserFom褦ˤޤQuickCheckenableQuickCheck, disableQuickCheckOn/OffǤޤ
 *
 * Revision 1.3  2004/06/30 16:19:15  yusuke
 *
 * iterative parse 
 *
 * Revision 1.2  2004/06/30 13:38:15  ninomi
 * ʸĹsetLimitWordsǡ票åsetLimitEdgesǥѡǤʤ
 * ФĤǤѹǤ褦ˤޤ
 *
 * Revision 1.1  2004/06/30 07:50:09  ninomi
 * up 0.2release versionǤ
 *
 */

