/*
 * $Id: ParserCkyIter.cc,v 1.3 2005/06/21 10:17:12 yusuke Exp $
 *
 *  Copyright (c) 2005, Tsujii Laboratory, The University of Tokyo.
 *  All rights reserved.
 *
 */

#include "ParserCkyIter.h"

namespace up {
    using namespace lilfes;

    bool ParserCkyIter::parsePhrase() {
        if(! isIterativeParse() ) { return ParserCkyFom::parsePhrase(); }
        
        int beam_num = beam_threshold_num_start;
        double beam_width = beam_threshold_width_start;
        double global_beam_width = global_beam_threshold_width_start;
        while ( true ) {
                //std::cerr << "beam=(" << beam_num << ',' << beam_width << ')' << std::endl;
            if ( beam_num > beam_threshold_num_end ||
                 beam_width > beam_threshold_width_end ||
                 global_beam_width > global_beam_threshold_width_end ) return true;
            setBeamThresholdNum( beam_num );
            setBeamThresholdWidth( beam_width );
            setGlobalBeamThresholdWidth( global_beam_width );
            
            if ( ! ParserCkyFom::parsePhrase() ) return false;

                // check root condition
            std::vector<up::Edge*> &ee = getEdges(0, getSentenceLength());
            std::vector<up::Edge*>::iterator it = ee.begin(), last = ee.end();

            {
                IPTrailStack iptrail(mach);
                for(; it != last ; ++it) {
                    FSP root = FSP(mach, (*it)->getSign());
                    if ( grammar->rootSign( root ) )
                        return true;
                }
            }

                // retry with wider thresholds
            beam_num += beam_threshold_num_step;
            beam_width += beam_threshold_width_step;
            global_beam_width += global_beam_threshold_width_step;
            if ( beam_num > beam_threshold_num_end ||
                 beam_width > beam_threshold_width_end ||
                 global_beam_width > global_beam_threshold_width_end) return true;
            
            if(! isReuseIterativeParse() ) {
                clear();
                if ( ! parseLex(getWordLattice()) ) return false;
            }
            
            incCurrentState();
        }
    }
    
} // end of namespace up

/*
 * $Log: ParserCkyIter.cc,v $
 * Revision 1.3  2005/06/21 10:17:12  yusuke
 *
 * copyright 񤭤ޤ
 *
 * Revision 1.2  2005/06/16 07:25:36  yusuke
 *
 * integrate-up 硥
 * version 2.0
 *
 * Revision 1.1.2.3  2005/06/06 08:53:41  ninomi
 * IPTrailStackطľޤlilfespredicate callʤޤ
 * quick checkidҤˤΤnameǤҤ褦ˤޤ
 *
 * Revision 1.1.2.2  2005/06/05 14:10:48  ninomi
 * ѥ뤬Ȥ褦ˤʤޤ
 *
 * Revision 1.1.2.1  2005/06/05 12:19:49  ninomi
 * upmayzβˤĤޤ
 *
 * Revision 1.15  2005/06/03 04:58:38  ninomi
 * Ȥꤢglobal thresholdingǤreleaseǤ
 *
 * Revision 1.14  2005/06/02 08:41:27  ninomi
 * localthresholding on/offäޤɤޤ
 *
 * Revision 1.13  2005/06/01 08:53:26  ninomi
 * ɤbug fix
 *
 * Revision 1.12  2005/06/01 07:38:02  ninomi
 * ̤iterative+global thresholding left cornerȤȤȤޤΤľޤ
 *
 * Revision 1.11  2005/06/01 05:36:28  ninomi
 * ParserCkyReuseIterglobal thresholding, global thresholding light, global thresholding left cornerбޤ
 *
 * Revision 1.10  2005/05/31 13:40:20  ninomi
 * global thresholding light ޤ
 *
 * Revision 1.9  2005/05/31 12:21:07  ninomi
 * global thresholding algorithmɲäޤ
 *
 * Revision 1.8  2005/05/29 10:51:07  ninomi
 * iterative parsingǳiterationclearΤǤϤʤiterationη̤Ѥversion (ParserCkyReuseIter)
 *
 * Revision 1.7  2005/05/20 03:38:05  yusuke
 *
 * Ϥ˼ԤȤλ chart Ĥޤޤˤ褦ˤ
 *
 * Revision 1.6  2005/04/25 11:48:23  yusuke
 *
 * ʸˡĤФʤȤϡparse/1  fail ʤ褦ˤ
 * parse/1  fail Τϡedge limit ۤȤ
 *
 * Revision 1.5  2004/11/16 09:14:07  ninomi
 * Artistic LicenseƬˤĤޤ
 *
 * Revision 1.4  2004/07/13 14:41:08  yusuke
 *
 * Ҥ beam width ǤѡʤȤchart  clear 褦ˤ
 *
 * Revision 1.3  2004/07/09 02:11:54  yusuke
 *
 * Parser::getWordLattice 
 * iterative parsing ǡbeam ѤƤʤȤ
 * parseLexiconLookup ʤ褦ˤ
 *
 * Revision 1.2  2004/07/06 13:05:57  ninomi
 * Parser::term()ɲäޤlilfesterminator::PerformAll()¹
 * ɬ¹ԤʤȤʤ
 *
 * Revision 1.1  2004/07/02 08:07:45  ninomi
 * ParserCkyFomiterative parseεǽParserCkyIter˰ܤޤ
 *
 */
