/*
 * $Id: ParserCkyIter.h,v 1.3 2005/06/21 10:17:12 yusuke Exp $
 *
 *  Copyright (c) 2005, Tsujii Laboratory, The University of Tokyo.
 *  All rights reserved.
 *
 */

#ifndef __ParserCkyIter_h
#define __ParserCkyIter_h

#include "ParserCkyFom.h"
#include "Fom.h"
#include <vector>
#include <liblilfes/machine.h>
#include <liblilfes/ftype.h>
#include <liblilfes/proc.h>
#include <liblilfes/structur.h>

namespace up {
    class ParserCkyIter : public ParserCkyFom {
    protected:
        // for iterative beam search
        bool iterative_parse;
        bool reuse_iterative_parse;
        
        int beam_threshold_num_start;
        int beam_threshold_num_step;
        int beam_threshold_num_end;
        
        double beam_threshold_width_start;
        double beam_threshold_width_step;
        double beam_threshold_width_end;
        
        double global_beam_threshold_width_start;
        double global_beam_threshold_width_end;
        double global_beam_threshold_width_step;
        
    protected:
        bool parsePhrase();
    public:
        ParserCkyIter()
                :
                iterative_parse(false),
                reuse_iterative_parse(false),
                beam_threshold_num_start(12),
                beam_threshold_num_step(6),
                beam_threshold_num_end(30),
                beam_threshold_width_start(6.0),
                beam_threshold_width_step(3.0),
                beam_threshold_width_end(15.0),
                global_beam_threshold_width_start(6.0),
                global_beam_threshold_width_end(15.0),
                global_beam_threshold_width_step(3.0) {}
        ~ParserCkyIter() { term(); }

        bool isIterativeParse() const { return iterative_parse; }
        void enableIterativeParse( int num_start, double width_start, double global_width_start,
                                   int num_end, double width_end, double global_width_end,
                                   int num_step, double width_step, double global_width_step ) {
          iterative_parse = true;
          beam_threshold_num_start = num_start;
          beam_threshold_width_start = width_start;
          global_beam_threshold_width_start = global_width_start;
          beam_threshold_num_end = num_end;
          beam_threshold_width_end = width_end;
          global_beam_threshold_width_end = global_width_end;
          beam_threshold_num_step = num_step;
          beam_threshold_width_step = width_step;
          global_beam_threshold_width_step = global_width_step;
        }
        void enableIterativeParse( int num_start, double width_start,
                                   int num_end, double width_end,
                                   int num_step, double width_step ) {
          iterative_parse = true;
          beam_threshold_num_start = num_start;
          beam_threshold_width_start = width_start;
          beam_threshold_num_end = num_end;
          beam_threshold_width_end = width_end;
          beam_threshold_num_step = num_step;
          beam_threshold_width_step = width_step;
        }
        void disableIterativeParse() { iterative_parse = false; }
        
        bool isReuseIterativeParse() const { return reuse_iterative_parse; }
        void enableReuseIterativeParse() { reuse_iterative_parse = true; }
        void disableReuseIterativeParse() { reuse_iterative_parse = false; }
    };
}
#endif // __ParserCkyIter_h

/*
 * $Log: ParserCkyIter.h,v $
 * Revision 1.3  2005/06/21 10:17:12  yusuke
 *
 * copyright 񤭤ޤ
 *
 * Revision 1.2  2005/06/16 07:25:36  yusuke
 *
 * integrate-up 硥
 * version 2.0
 *
 * Revision 1.1.2.1  2005/06/05 12:19:49  ninomi
 * upmayzβˤĤޤ
 *
 * Revision 1.9  2005/06/03 04:58:32  ninomi
 * Ȥꤢglobal thresholdingǤreleaseǤ
 *
 * Revision 1.8  2005/06/02 08:41:21  ninomi
 * localthresholding on/offäޤɤޤ
 *
 * Revision 1.7  2005/06/01 05:36:20  ninomi
 * ParserCkyReuseIterglobal thresholding, global thresholding light, global thresholding left cornerбޤ
 *
 * Revision 1.6  2005/05/31 13:40:11  ninomi
 * global thresholding light ޤ
 *
 * Revision 1.5  2005/05/31 12:20:57  ninomi
 * global thresholding algorithmɲäޤ
 *
 * Revision 1.4  2005/05/29 10:50:58  ninomi
 * iterative parsingǳiterationclearΤǤϤʤiterationη̤Ѥversion (ParserCkyReuseIter)
 *
 * Revision 1.3  2004/11/16 09:13:54  ninomi
 * Artistic LicenseƬˤĤޤ
 *
 * Revision 1.2  2004/07/06 13:05:50  ninomi
 * Parser::term()ɲäޤlilfesterminator::PerformAll()¹
 * ɬ¹ԤʤȤʤ
 *
 * Revision 1.1  2004/07/02 08:07:37  ninomi
 * ParserCkyFomiterative parseεǽParserCkyIter˰ܤޤ
 *
 */

