/*
 * $Id: PreLex.h,v 1.2 2005/06/26 09:05:59 ninomi Exp $
 *
 *  Copyright (c) 2005, Tsujii Laboratory, The University of Tokyo.
 *  All rights reserved.
 */

#ifndef __PreLex_h
#define __PreLex_h

#include <liblilfes/structur.h>
#include <utility>
#include "Fs.h"

namespace up {
    
    typedef lilfes::uint32 pserial;

    class PreLex {
    private:
        pserial serial_no;
        ExtentWord *extent;
        Fs* lexid;
        Fs* word;
        double fom;
    public:
        PreLex(pserial __serial_no, ExtentWord *__extent, Fs* __lexid, Fs* __word, double __fom)
                : serial_no(__serial_no), extent(__extent), lexid(__lexid), word(__word), fom(__fom) {}
        ~PreLex() {
            if(lexid) delete lexid;
//            if(word) delete word;
        }
        pserial getSerialNo() const { return serial_no; }
        void setSerialNo(pserial psn) { serial_no = psn; }
        ExtentWord* getExtentWord() const { return extent; }
        void setExtentWord(ExtentWord *exw) { extent = exw; }
        Fs* getLexID() const { return lexid; }
        void setLexID(Fs* l) { lexid = l; }
        Fs* getWord() const { return word; }
        void setWord(Fs* w) { word = w; }
        double getFom() const { return fom; }
        void setFom(double f) { fom = f; }
    };

    class PreLexManager {
    private:
        std::vector<PreLex*> serialtable;
    public:
        PreLexManager() {}
        ~PreLexManager() {
            std::vector<PreLex*>::iterator it = serialtable.begin(), last = serialtable.end();
            for( ; it != last ; it++)
                delete(*it);
        }
        PreLex *makePreLex(ExtentWord *exw, Fs *lexid, Fs* word, double fom) {
            PreLex *p = new PreLex((pserial) serialtable.size(), exw, lexid, word, fom);
            serialtable.push_back(p);
            return p;
        }
        PreLex *getPreLex(pserial psn) {
            if(psn >= serialtable.size())
                return (PreLex *) 0;
            else
                return serialtable[psn];
        }
        pserial getSerialCount() { return (pserial) serialtable.size(); }
        void clear() {
            std::vector<PreLex*>::iterator it = serialtable.begin(), last = serialtable.end();
            for( ; it != last ; it++)
                delete (*it);
            serialtable.clear();
        }
    };
}

#endif // __PreLex_h

/*
 * $Log: PreLex.h,v $
 * Revision 1.2  2005/06/26 09:05:59  ninomi
 * PreLex򤤤줿С
 *
 * Revision 1.1  2005/06/25 10:25:09  ninomi
 * PreLex.h is added.
 * k  Lines beginning with `CVS:' are removed automatically
 *
 */
