/*
 * $Id: Qc.h,v 1.3 2005/06/21 10:17:12 yusuke Exp $
 *
 *  Copyright (c) 2005, Tsujii Laboratory, The University of Tokyo.
 *  All rights reserved.
 *
 */

#ifndef __Qc_h
#define __Qc_h

#include <liblilfes/proc.h>
#include <liblilfes/utility.h>
#include "Edge.h"
#include "Extent.h"
#include "QuickCheck.h"
#include "Grammar.h"


namespace up {

    class QcManager {
    private:
        bool is_open;
	lilfes::machine* mach;
	qc::QuickCheckManager* qcm;
	qc::QuickCheckArrayManager* qcam;
	Grammar* grammar;
	std::vector<std::pair<lilfes::code*, lilfes::core_p> > stack;
        std::vector<qc::QCA*> edge_qca;
        std::vector<std::vector<std::pair<Fs*, qc::QCA*> > > idschema_qca;

    public:
        QcManager() : qcm(0), qcam(0) {}
        ~QcManager() { term(); }
        bool isOpen() { return is_open; }
	qc::QCA* getEdgeQca(eserial esn) { return edge_qca[esn]; }
	std::vector<std::pair<Fs*, qc::QCA*> >& getIdSchemaQca(eserial esn) {
	    return idschema_qca[esn];
	}
        bool isValid(eserial esn) { return (esn < edge_qca.size() && edge_qca[esn] != 0); }
	void makeQca(eserial esn, Fs* x);
	bool quickCheck(qc::QCA* x, qc::QCA* y);
	void clear();
        bool setQuickCheckPath(lilfes::FSP paths) {
            if(!is_open) return false;
            return qcm->Init(paths);
        }
        bool init(lilfes::machine* m, Grammar* g) {
	    mach = m; grammar = g;
	    qcm = new qc::QuickCheckManager();
	    qcam = new qc::QuickCheckArrayManager(qcm);
            is_open = true;
	    return true;
	}
        bool term() {
            if(is_open) {
                if(qcm) delete qcm;
                if(qcam) delete qcam;
            }
            is_open = false;
            return true;
        }
    };
}

#endif // __Qc_h

/*
 * $Log: Qc.h,v $
 * Revision 1.3  2005/06/21 10:17:12  yusuke
 *
 * copyright 񤭤ޤ
 *
 * Revision 1.2  2005/06/16 07:25:36  yusuke
 *
 * integrate-up 硥
 * version 2.0
 *
 * Revision 1.1.2.7  2005/06/08 05:59:42  ninomi
 * P2ץեĤʤޤ
 *
 * Revision 1.1.2.6  2005/06/06 10:39:54  ninomi
 * quick check pathꡢʬ񤭤ޤ
 *
 * Revision 1.1.2.5  2005/06/06 08:53:41  ninomi
 * IPTrailStackطľޤlilfespredicate callʤޤ
 * quick checkidҤˤΤnameǤҤ褦ˤޤ
 *
 * Revision 1.1.2.4  2005/06/05 14:10:48  ninomi
 * ѥ뤬Ȥ褦ˤʤޤ
 *
 * Revision 1.1.2.3  2005/06/05 13:14:21  ninomi
 * Ȥꤢ񴹤ޤǤФäϤ줫
 *
 * Revision 1.1.2.2  2005/06/05 12:19:49  ninomi
 * upmayzβˤĤޤ
 *
 * Revision 1.1.2.1  2005/06/05 10:12:19  ninomi
 * upmayz/srcβ˰ܤϤޤ
 *
 * Revision 1.3  2005/06/03 15:31:12  ninomi
 * ParserCkyQcβɤ򤷤ޤ
 *
 * Revision 1.2  2004/11/16 09:13:54  ninomi
 * Artistic LicenseƬˤĤޤ
 *
 * Revision 1.1  2004/06/30 07:50:09  ninomi
 * up 0.2release versionǤ
 *
 */
