/**********************************************************************
 *
 *  Copyright (c) 2005, Tsujii Laboratory, The University of Tokyo.
 *  All rights reserved.
 *
 *  @file TemplateTable.h
 *  @version Time-stamp: <2005-06-21 19:00:15 yusuke>
 *  Mappings from template names to signs
 *
 **********************************************************************/

#ifndef MAYZ_TEMPLATE_TABLE_H

#define MAYZ_TEMPLATE_TABLE_H

#include "config_mayz.h"
#include "FSArray.h"
#include "LexEntry.h"
#include <liblilfes/builtin.h>
#include <string>
#include <vector>

namespace mayz {

  class TemplateTable {
  private:
    lilfes::machine* mach;
    FSArray< lilfes::FSP, lilfes::FSP > template_db;
    FSArray< lilfes::FSP, int > count_db;

  public:
    typedef FSArray< lilfes::FSP, lilfes::FSP >::iterator iterator;
    iterator begin() { return template_db.begin(); }
    iterator end() { return template_db.end(); }

  public:
    TemplateTable( lilfes::machine* m ) : mach( m ), template_db( mach ), count_db( mach ) {
    }
    virtual ~TemplateTable() {}

    bool incCount( const LexTemplate& name, int i = 1 ) {
      int count = 0;
      if ( ! count_db.find( name, count ) ) {
        count = 0;
      }
      count += i;
      return count_db.insert( name, count );
    }
    bool getCount( const LexTemplate& name, int& count ) {
      return count_db.find( name, count );
    }
//     virtual bool newTemplate( const LexTemplate& name, lilfes::FSP temp ) {
//       int count = 0;
//       if ( count_db.find( name, count ) ) {
//         return count_db.insert( name, ++count );
//       } else {
//         return count_db.insert( name, 1 )
//           && template_db.insert( name, temp );
//       }
//     }
    virtual bool insert( const LexTemplate& name, lilfes::FSP temp ) {
      return template_db.insert( name, temp );
    }
    virtual bool find( const LexTemplate& name, lilfes::FSP temp ) {
      lilfes::FSP t;
      return template_db.find( name, t ) && temp.Unify( t );
    }

    virtual bool load( const std::string& lexfile, const std::string& countfile = "" );
    virtual bool save( const std::string& lexfile, const std::string& countfile = "" );
  };

}

#endif // MAYZ_TEMPLATE_TABLE_H
