/**********************************************************************
 *
 *  Copyright (c) 2005, Tsujii Laboratory, The University of Tokyo.
 *  All rights reserved.
 *
 *  @file WordCounter.cc
 *  @version Time-stamp: <2005-06-21 19:01:58 yusuke>
 *  A class for counting frequency of words
 *
 **********************************************************************/

#include "WordCounter.h"
#include <liblilfes/builtin.h>

#define WORD_COUNT_KEY "word_count_key"
static lilfes::procedure* word_count_key = NULL;

namespace mayz {

  using namespace lilfes;
  using namespace std;

  bool WordCounter::getWordKey( FSP lexicon_key, FSP& word_key ) {
    if ( key_map_db.find( lexicon_key, word_key ) ) {
      return true;
    }
    vector< FSP > args( 2 );
    args[ 0 ] = lexicon_key;
    args[ 1 ] = FSP( mach );
    core_p cutp = mach->GetCutPoint();
    if ( ! call_proc( *mach, word_count_key, args ) ) {
      return false;
    }
    mach->DoCut( cutp );
    word_key = args[ 1 ];
    return key_map_db.insert( lexicon_key, word_key );
  }

  void WordCounter::initialize() {
    word_count_key = prepare_proc( module::UserModule(), WORD_COUNT_KEY, 2 );
    if ( ! word_count_key ) {
      throw WordCounterException( "Predicate " WORD_COUNT_KEY "/2 not found" );
    }
  }

  bool WordCounter::incCount( FSP lexicon_key, int c ) {
    FSP word_key;
    if ( ! getWordKey( lexicon_key, word_key ) ) return false;
    int word_count = 0;
    if ( ! count_db.find( word_key, word_count ) ) {
      word_count = 0;
    }
    word_count += c;
    return count_db.insert( word_key, word_count );
  }

  bool WordCounter::getCount( FSP lexicon_key, int& c ) {
    FSP word_key;
    if ( ! getWordKey( lexicon_key, word_key ) ) return false;
    return count_db.find( word_key, c );
  }

}

