/**********************************************************************
 *
 *  Copyright (c) 2005, Tsujii Laboratory, The University of Tokyo.
 *  All rights reserved.
 *
 *  @file WordCounter.h
 *  @version Time-stamp: <2005-06-21 19:02:06 yusuke>
 *  A class for counting frequency of words
 *
 **********************************************************************/

#ifndef MAYZ_WORD_COUNTER_H

#define MAYZ_WORD_COUNTER_H

#include "config_mayz.h"
#include "FSArray.h"
#include <liblilfes/builtin.h>
#include <string>
#include <vector>

namespace mayz {

  class WordCounterException : public std::exception {
  private:
    std::string message;
  public:
    WordCounterException( const char* m ) : message( m ) {}
    WordCounterException( const std::string& m ) : message( m ) {}
    ~WordCounterException() throw () {}
    const char* what() const throw () { return message.c_str(); }
  };

  class WordCounter {
  private:
    lilfes::machine* mach;
    FSArray< lilfes::FSP, int > count_db;
    FSArray< lilfes::FSP, lilfes::FSP > key_map_db;

  public:
    typedef FSArray< lilfes::FSP, int >::iterator iterator;
    iterator begin() { return count_db.begin(); }
    iterator end() { return count_db.end(); }

  protected:
    bool getWordKey( lilfes::FSP lexicon_key, lilfes::FSP& word_key );

  public:
    WordCounter( lilfes::machine* m ) : mach( m ), count_db( mach ), key_map_db( mach ) {
    }
    virtual ~WordCounter() {}

    virtual void initialize();

    virtual bool incCount( lilfes::FSP lexicon_key, int c = 1 );
    virtual bool getCount( lilfes::FSP lexicon_key, int& count );
  };

}

#endif // MAYZ_WORD_COUNTER_H
